/*
*  qm_browser.cpp
*  QUIMUP library & playlist window
*  (c) 2008-2010 Johan Spee
*
*  This file is part of Quimup
*
*  QUIMUP is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  QUIMUP is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program. If not, see http://www.gnu.org/licenses/.
*/

#include "qm_browser.h"

qm_browser::qm_browser()
{
    mpd_connected = false;
    b_startup = true;
    setupUI();
}


void qm_browser::setupUI()
{
    if (objectName().isEmpty())
        setObjectName("browserWin");
        
    QPixmap qpx;
    qpx = QPixmap(":/mn_icon.png");
    setWindowIcon(qpx);
    setWindowTitle(trUtf8("Quimup Browser"));
    
    QSizePolicy sizePolicy(static_cast<QSizePolicy::Policy>(5), static_cast<QSizePolicy::Policy>(5));
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(this->sizePolicy().hasHeightForWidth());
    setSizePolicy(sizePolicy);
    setMinimumSize(QSize(600, 400));
    setMaximumSize(QSize(32767, 32767));
    
    main_widget = new QWidget();
    vbox_all = new QVBoxLayout(main_widget);
    setCentralWidget(main_widget); // 'attatches' it to the window
    vbox_all->setMargin(4);
    vbox_all->setSpacing(0);
    
    theSplitter = new QSplitter(this);
    theSplitter->setOrientation(Qt::Horizontal);
    theSplitter->setHandleWidth(10);
    
    left_group = new QLabel(theSplitter);
    left_group->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
    vbox_lefthalf = new QVBoxLayout(left_group);
    vbox_lefthalf->setSpacing(4);
    vbox_lefthalf->setMargin(2);
    lb_left_title = new QLabel;
    lb_left_title->setTextFormat(Qt::RichText);
    lb_left_title->setAlignment(Qt::AlignHCenter);
    lb_left_title->setAutoFillBackground(true);
    lb_left_title->setFrameShape(QFrame::NoFrame);
    lb_left_title->setFrameShadow(QFrame::Raised);
    vbox_lefthalf->addWidget(lb_left_title);
    hbox_leftop = new QHBoxLayout();
    hbox_leftop->setSpacing(4);
    hbox_leftop->setMargin(0);
    cb_dbasemode = new QComboBox();
    cb_dbasemode->setMinimumSize(QSize(100, 26));
    hbox_leftop->addWidget(cb_dbasemode);
    spacer1 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    hbox_leftop->addItem(spacer1);
    bt_LIBRARY = new QPushButton();
    bt_LIBRARY->setMinimumSize(QSize(0, 26));
    bt_LIBRARY->setIcon(QIcon(":/tr_expand1.png"));
    bt_LIBRARY->setFlat(true);
    hbox_leftop->addWidget(bt_LIBRARY);
    spacer2 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    hbox_leftop->addItem(spacer2);
    bt_update = new QPushButton();
    bt_update->setMinimumSize(QSize(100, 26));
    bt_update->setIcon(QIcon(":/tr_update.png"));
    hbox_leftop->addWidget(bt_update);
    vbox_lefthalf->addLayout(hbox_leftop);
    lib_view = new qm_libview(left_group);
    lib_view->setHorizontalScrollBarPolicy (Qt::ScrollBarAlwaysOn);
    lib_view->setVerticalScrollBarPolicy (Qt::ScrollBarAlwaysOn); // avoid problems with viewport()->width();
    lib_view->setSizePolicy (QSizePolicy::Expanding, QSizePolicy::Expanding);
    lib_view->setUniformRowHeights(true);
    lib_view->setAlternatingRowColors(true);
    vbox_lefthalf->addWidget(lib_view);
    lb_dbasestats = new QLabel();
    lb_dbasestats->setAlignment(Qt::AlignHCenter);
    lb_dbasestats->setAutoFillBackground(true);
    vbox_lefthalf->addWidget(lb_dbasestats);
    hbox_leftbottom = new QHBoxLayout();
    hbox_leftbottom->setSpacing(4);
    hbox_leftbottom->setMargin(0);
    cb_searchmode = new QComboBox();
    cb_searchmode->setMinimumSize(QSize(100, 26));
    hbox_leftbottom->addWidget(cb_searchmode);
    le_search = new QLineEdit();
    hbox_leftbottom->addWidget(le_search);
    bt_search = new QPushButton();
    bt_search->setMinimumSize(QSize(100, 26));
    bt_search->setIcon(QIcon(":/tr_pointleft.png"));
    hbox_leftbottom->addWidget(bt_search);
    vbox_lefthalf->addLayout(hbox_leftbottom);
    theSplitter->addWidget(left_group);
    
    right_group = new QLabel(theSplitter);
    right_group->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
    vbox_righthalf = new QVBoxLayout(right_group);
    vbox_righthalf->setSpacing(4);
    vbox_righthalf->setMargin(2);
    lb_right_title = new QLabel;
    lb_right_title->setTextFormat(Qt::RichText);
    lb_right_title->setAlignment(Qt::AlignHCenter);
    lb_right_title->setAutoFillBackground(true);
    lb_right_title->setFrameShape(QFrame::NoFrame);
    lb_right_title->setFrameShadow(QFrame::Raised);
    vbox_righthalf->addWidget(lb_right_title);
    hbox_rightop = new QHBoxLayout();
    hbox_rightop->setSpacing(4);
    hbox_rightop->setMargin(0);
    bt_stream = new QPushButton();
    bt_stream->setMinimumSize(QSize(100, 26));
    bt_stream->setIcon(QIcon(":/tr_stream.png"));
    hbox_rightop->addWidget(bt_stream);
    spacer3 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    hbox_rightop->addItem(spacer3);
    bt_PLAYLIST = new QPushButton();
    bt_PLAYLIST->setMinimumSize(QSize(0, 26));
    bt_PLAYLIST->setIcon(QIcon(":/tr_expand1.png"));
    bt_PLAYLIST->setFlat(true);
    hbox_rightop->addWidget(bt_PLAYLIST);
    spacer4 = new QSpacerItem( 0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum );
    hbox_rightop->addItem(spacer4);
    bt_options = new QPushButton();
    bt_options->setMinimumSize(QSize(100, 26));
    bt_options->setIcon(QIcon(":/tr_options.png"));
    hbox_rightop->addWidget(bt_options);
    vbox_righthalf->addLayout(hbox_rightop);
    plist_view = new qm_plistview(right_group);
    plist_view->setHorizontalScrollBarPolicy (Qt::ScrollBarAlwaysOn);
    plist_view->setVerticalScrollBarPolicy (Qt::ScrollBarAlwaysOn); // avoid problems with viewport()->width();
    plist_view->setSizePolicy (QSizePolicy::Expanding, QSizePolicy::Expanding);
    plist_view->setUniformRowHeights(true);
    plist_view->setAlternatingRowColors(true);
    vbox_righthalf->addWidget(plist_view);
    lb_pliststats = new QLabel();
    lb_pliststats->setAlignment(Qt::AlignHCenter);
    lb_pliststats->setAutoFillBackground(true);
    vbox_righthalf->addWidget(lb_pliststats);
    hbox_rightbottom = new QHBoxLayout();
    hbox_rightbottom->setSpacing(4);
    hbox_rightbottom->setMargin(0);
    cb_selectmode = new QComboBox();
    cb_selectmode->setMinimumSize(QSize(100, 26));
    hbox_rightbottom->addWidget(cb_selectmode);
    le_select = new QLineEdit();
    hbox_rightbottom->addWidget(le_select);
    bt_select = new QPushButton();
    bt_select->setMinimumSize(QSize(100, 26));
    bt_select->setIcon(QIcon(":/tr_pointleft.png"));
    hbox_rightbottom->addWidget(bt_select);
    vbox_righthalf->addLayout(hbox_rightbottom);
    theSplitter->addWidget(right_group);
    
    vbox_all->addWidget(theSplitter);
    
    cb_dbasemode->addItem ( QIcon(":/tr_artist.png"), trUtf8("Artists"));
    cb_dbasemode->addItem ( QIcon(":/tr_album.png"), trUtf8("Albums"));
    cb_dbasemode->addItem ( QIcon(":/tr_albumdate.png"), trUtf8("Modified"));
	cb_dbasemode->addItem ( QIcon(":/tr_albumyear.png"), trUtf8("Year"));
    cb_dbasemode->addItem ( QIcon(":/tr_genre.png"), trUtf8("Genres"));
    cb_dbasemode->addItem ( QIcon(":/tr_playlist.png"), trUtf8("Playlists"));
    cb_dbasemode->addItem ( QIcon(":/tr_folder.png"), trUtf8("Folders"));
    cb_dbasemode->addItem ( QIcon(":/tr_search.png"), trUtf8("Searched"));
    cb_dbasemode->setIconSize(QSize(16, 16));
    
	cb_searchmode->addItem ( QIcon(":/tr_dotsmall.png"), trUtf8("Artist"));
    cb_searchmode->addItem ( QIcon(":/tr_dotsmall.png"), trUtf8("Album"));
    cb_searchmode->addItem ( QIcon(":/tr_dotsmall.png"), trUtf8("Title"));
    cb_searchmode->addItem ( QIcon(":/tr_dotsmall.png"), trUtf8("Genre"));
    cb_searchmode->setIconSize(QSize(10, 10));
	
	cb_selectmode->addItem ( QIcon(":/tr_dotsmall.png"), trUtf8("Artist"));
    cb_selectmode->addItem ( QIcon(":/tr_dotsmall.png"), trUtf8("Album"));
    cb_selectmode->addItem ( QIcon(":/tr_dotsmall.png"), trUtf8("Title"));
    cb_selectmode->addItem ( QIcon(":/tr_dotsmall.png"), trUtf8("All"));
    cb_selectmode->setIconSize(QSize(10, 10));
	
    sloader = new qm_stream_loader();
    
    options_menu = new QMenu();
    
    a_random = new QAction(options_menu);
    a_random->setText(trUtf8("Random playback"));
    a_random->setCheckable(true);
    a_random->setIcon(QIcon(":/tr_random.png"));
    QObject::connect( a_random, SIGNAL( triggered() ), this, SLOT( set_random() ) );
    options_menu->addAction(a_random);
    
    a_repeat = new QAction(options_menu);
    a_repeat->setText(trUtf8("Repeat (list or single)"));
    a_repeat->setCheckable(true);
    a_repeat->setIcon(QIcon(":/tr_repeat.png"));
    QObject::connect( a_repeat, SIGNAL( triggered() ), this, SLOT( set_repeat() ) );
    options_menu->addAction(a_repeat);
    
    a_single = new QAction(options_menu);
    a_single->setText(trUtf8("Single mode"));
    a_single->setCheckable(true);
    a_single->setIcon(QIcon(":/tr_single.png"));
    QObject::connect( a_single, SIGNAL( triggered() ), this, SLOT( set_single() ) );
    options_menu->addAction(a_single);
    
    a_consume = new QAction(options_menu);
    a_consume->setText(trUtf8("Consume mode"));
    a_consume->setCheckable(true);
    a_consume->setIcon(QIcon(":/tr_consume.png"));
    QObject::connect( a_consume, SIGNAL( triggered() ), this, SLOT( set_consume() ) );
    options_menu->addAction(a_consume);
    
    a_xfade = new QAction(options_menu);
    a_xfade->setText(trUtf8("Cross fade"));
    a_xfade->setCheckable(true);
    a_xfade->setIcon(QIcon(":/tr_xfade.png"));
    QObject::connect( a_xfade, SIGNAL( triggered() ), this, SLOT( set_xfade() ) );
    options_menu->addAction(a_xfade);
    
    QObject::connect(sloader, SIGNAL(streamdownload_done(QStringList)), this, SLOT(on_streamdownload_done(QStringList)));
    QObject::connect(cb_dbasemode, SIGNAL(activated(int)), this, SLOT(on_dbase_newmode(int)));
    QObject::connect(cb_searchmode, SIGNAL(activated(int)), this, SLOT(set_searchmode(int)));
    QObject::connect(cb_selectmode, SIGNAL(activated(int)), this, SLOT(set_selectmode(int)));
    QObject::connect(bt_search, SIGNAL(clicked()), this, SLOT(search_in_lib()));
    QObject::connect(le_search, SIGNAL(returnPressed()), this, SLOT(search_in_lib()));
    QObject::connect(bt_select, SIGNAL(clicked()), this, SLOT(select_in_plist()));
    QObject::connect(le_select, SIGNAL(returnPressed()), this, SLOT(select_in_plist()));
    QObject::connect(bt_update, SIGNAL(clicked()), this, SLOT(on_update_dbase()));
    QObject::connect(bt_stream, SIGNAL(clicked()), this, SLOT(get_the_stream()));
    QObject::connect(bt_options, SIGNAL(clicked()), this, SLOT(on_options_clicked()));
    QObject::connect(plist_view, SIGNAL(sgnl_plist_statistics(QString)), this, SLOT(set_plist_stats(QString)));
    QObject::connect(plist_view, SIGNAL(sgnl_plist_dropreceived(int)), this, SLOT(on_plist_dropreceived(int)));
    QObject::connect(lib_view, SIGNAL(sgnl_newmode(int)), this, SLOT(on_lib_newmode(int)));
    QObject::connect(lib_view, SIGNAL(sgnl_playlist_modification(bool)), this, SLOT(on_playlist_modification(bool)));
    QObject::connect(bt_LIBRARY, SIGNAL(clicked()), this, SLOT(on_library_toggle()));
    QObject::connect(bt_PLAYLIST, SIGNAL(clicked()), this, SLOT(on_playlist_toggle()));
    QObject::connect(theSplitter, SIGNAL( splitterMoved( int, int )), this, SLOT( splittermoved() ));
    
    /*  plist_view and lib_view share thePlaylistModList
     for playlist modifications like drag&drop operations. */
    lib_view->set_playlistModList( &thePlaylistModList );
    plist_view->set_playlistModList( &thePlaylistModList );
    
} // setupUi


void qm_browser::get_the_stream()
{
    QString suggested_name = "http://";
    bool ok;
    QString input = QInputDialog::getText(this, trUtf8("Save"),
                                          trUtf8(" URL for the stream's playlist file: "), QLineEdit::Normal,
                                          suggested_name, &ok);
                                          
    if ( !ok || input.isEmpty() || input == "http://")
        return;
        
    if (!input.startsWith("http://"))
    {
        QMessageBox::warning( this, trUtf8("Error"), "URL must begin with \"http://\"");
        return;
    }
    
    sloader->download_this(input);
    // and wait for sgnl_on_streamdownload_done
}


void qm_browser::on_streamdownload_done(QStringList result)
{
    thePlaylistModList.clear();
    if (result.value(0) == "error")
    {
        QMessageBox::warning( this, trUtf8("Error"), result.value(1));
        return;
    }
    
    if (result.value(0) == "nothing")
    {
        QString message = trUtf8("Expected playlist file was not found.\nDownload aborted. ");
        QMessageBox::warning( this, trUtf8("Error"), message);
        return;
    }
    
    
    if (result.value(0) == "timeout")
    {
        QString message = trUtf8("Download takes too long (") + result.value(1) + trUtf8(" % in 10 sec.)\nDownload aborted. ");
        QMessageBox::warning( this, trUtf8("Error"), message);
        return;
    }
    
    int count = 0;
    
    for (int i = 2; i < result.size(); ++i)
    {
        qm_mpd_command newCommand;
        newCommand.cmd = CMD_ADD;
        newCommand.file = result.value(i);
        newCommand.moveTo = 0;
        newCommand.artist = "Stream ";
        newCommand.title = result.value(i);
        newCommand.time = 0;
        newCommand.album = "Info in player ";
        newCommand.track = "";
        newCommand.type = TP_STREAM;
        thePlaylistModList.push_back(newCommand);
        count++;
    }
    
    mpdCom->execute_cmds(thePlaylistModList, false );
    plist_view->on_playlist_modification(false);
    
    QString message = "Downloaded file type: " + result.value(1) + " \nAdded " + QString::number(count) + " items to the playlist. ";
    
    QMessageBox::information( this, trUtf8("Confirmation"), message);
}


void qm_browser::on_playlist_modification(bool dragdrop)
{
    plist_view->on_playlist_modification(dragdrop);
}

void qm_browser::on_plist_dropreceived(int pos)
{
    lib_view->on_plist_dropreceived(pos);
}

void qm_browser::on_lib_newmode(int newmode)
{
    cb_dbasemode->blockSignals(true);
    cb_dbasemode->setCurrentIndex(newmode);
    cb_dbasemode->blockSignals(false);
    // libview must know what to search for
    if (newmode == LB_ID_SEARCH)
        config->search_string = le_search->text();
}

void qm_browser::on_dbase_newmode(int i)
{

    config->browser_dbasemode = i;
    lib_view->reload_root();
}


void qm_browser::set_searchmode(int i)
{
    config->browser_searchmode = i;
}


void qm_browser::set_selectmode(int i)
{
    config->browser_selectmode = i;
}


void qm_browser::search_in_lib()
{
    config->search_string = le_search->text();
    cb_dbasemode->blockSignals(true);
    cb_dbasemode->setCurrentIndex(LB_ID_SEARCH);
    cb_dbasemode->blockSignals(false);
    on_dbase_newmode(LB_ID_SEARCH);
    lib_view->setFocus();
}


void qm_browser::select_in_plist()
{
    config->select_string = le_select->text();
    plist_view->select_it(cb_selectmode->currentIndex() );
    plist_view->setFocus();
}


void qm_browser::on_options_clicked()
{
    options_menu->exec(QCursor::pos());
}

void qm_browser::retranslateUi()
{
    if (config->show_tooltips)
    {
        bt_stream->setToolTip(trUtf8(" Enter a URL to an internet audio stream "));
        bt_options->setToolTip(trUtf8(" Set playback options "));
        bt_update->setToolTip(trUtf8(" Scan the entire music collection for changes "));
        cb_searchmode->setToolTip(trUtf8(" Search mode "));
        cb_selectmode->setToolTip(trUtf8(" Select mode "));
        cb_dbasemode->setToolTip(trUtf8(" Library mode "));
    }
    
    bt_select->setText(trUtf8("Select"));
    bt_search->setText(trUtf8("Search"));
    bt_update->setText(trUtf8("Update"));
    bt_options->setText(trUtf8("Options"));
    bt_stream->setText(trUtf8("Stream"));
    lb_left_title->setText(trUtf8("<b>Library</b>"));
    lb_right_title->setText(trUtf8("<b>Playlist</b>"));
}


void qm_browser::set_config(qm_config *conf)
{
    if (conf == NULL)
        return;
        
    config = conf;
    retranslateUi();
    cb_dbasemode->blockSignals(true);
    cb_dbasemode->setCurrentIndex(config->browser_dbasemode);
    cb_dbasemode->blockSignals(false);
    cb_searchmode->setCurrentIndex(config->browser_searchmode);
    if (config->browser_dbasemode != LB_ID_SEARCH)
        config->search_string = "";
    le_search->setText(config->search_string);
    cb_selectmode->setCurrentIndex(config->browser_selectmode);
    config->select_string = "";
    le_select->setText(config->select_string);
    
    // set the splitter pos
    QList<int> List;
    List.append(config->browser_L_splitter);
    List.append(config->browser_R_splitter);
    theSplitter->setSizes(List);
    
    resize(config->browserWin_W, config->browserWin_H );
}


void qm_browser::set_font(QFont & thefont)
{
    plist_view->set_font(thefont);
    lib_view->set_font(thefont);
}


void qm_browser::splittermoved()
{
    if (theSplitter->sizes().value(0) != 0 && theSplitter->sizes().value(1) != 0)
    {
        config->browser_L_splitter = theSplitter->sizes().value(0);
        config->browser_R_splitter = theSplitter->sizes().value(1);
    }
}


void qm_browser::hideEvent(QHideEvent *he)
{
    config->browserWin_X = x();
    config->browserWin_Y = y();
    config->browserWin_W = width();
    config->browserWin_H = height();
    he->accept();
}


void qm_browser::set_plist_stats(QString stats)
{
    lb_pliststats->setText(stats);
}


void qm_browser::set_dbase_stats(QString stats)
{
    lb_dbasestats->setText(stats);
}


void qm_browser::showEvent(QShowEvent *se)
{
    move(config->browserWin_X, config->browserWin_Y);
    se->accept();
}


void qm_browser::closeEvent( QCloseEvent *ce )
{
    this->hide();
    ce->ignore(); // only hide, don't close
}


void qm_browser::set_mpdCom(qm_mpdCom* conn, bool isconnected)
{
    mpd_connected = isconnected;
    mpdCom = conn;
    QObject::connect(mpdCom, SIGNAL( sgnl_dbase_statistics(QString)), this, SLOT(set_dbase_stats(QString)));
    QObject::connect(mpdCom, SIGNAL( sgnl_update_ready()), this, SLOT(on_update_ready()));
    mpdCom->get_statistics();
    
    if (!config->mpd_update_allowed)
        bt_update->setEnabled(false);
    else
        bt_update->setEnabled(true);
        
    if (config->mpd_single_allowed)
        a_single->setEnabled(true);
    else
        a_single->setEnabled(false);
        
    if (config->mpd_consume_allowed)
        a_consume->setEnabled(true);
    else
        a_consume->setEnabled(false);
        
    if (config->mpd_xfade_allowed)
        a_xfade->setEnabled(true);
    else
        a_xfade->setEnabled(false);
        
    if (config->mpd_repeat_allowed)
        a_repeat->setEnabled(true);
    else
        a_repeat->setEnabled(false);
        
    if (config->mpd_random_allowed)
        a_random->setEnabled(true);
    else
        a_random->setEnabled(false);
}


void qm_browser::on_update_dbase()
{
    if (!config->mpd_update_allowed)
        return;
        
    bt_update->setEnabled(false);
    bt_update->setIcon(QIcon(":/tr_dot.png"));
    mpdCom->update_all();
}


void qm_browser::on_update_ready()
{
    bt_update->setDisabled(false);
    bt_update->setIcon(QIcon(":/tr_update.png"));
    lib_view->reload_root();
}

void qm_browser::set_menu_random(bool isitso)
{
    a_random->blockSignals(true);
    a_random->setChecked(isitso);
    a_random->blockSignals(false);
}

void qm_browser::set_menu_repeat(bool isitso)
{
    a_repeat->blockSignals(true);
    a_repeat->setChecked(isitso);
    a_repeat->blockSignals(false);
}

void qm_browser::set_menu_single(bool isitso)
{
    a_single->blockSignals(true);
    a_single->setChecked(isitso);
    a_single->blockSignals(false);
}

void qm_browser::set_menu_consume(bool isitso)
{
    a_consume->blockSignals(true);
    a_consume->setChecked(isitso);
    a_consume->blockSignals(false);
}

void qm_browser::set_menu_xfade(bool isitso)
{
    a_xfade->blockSignals(true);
    a_xfade->setChecked(isitso);
    a_xfade->blockSignals(false);
}


void qm_browser::set_random()
{
    if (mpd_connected)
        mpdCom->set_random(a_random->isChecked());
    else
        set_menu_random(!a_random->isChecked());
}

void qm_browser::set_repeat()
{
    if (mpd_connected)
        mpdCom->set_repeat(a_repeat->isChecked());
    else
        set_menu_repeat(!a_repeat->isChecked());
}


void qm_browser::set_single()
{
    if (mpd_connected)
        mpdCom->set_single(a_single->isChecked());
    else
        set_menu_single(!a_single->isChecked());
}


void qm_browser::set_consume()
{
    if (mpd_connected)
        mpdCom->set_consume(a_consume->isChecked());
    else
        set_menu_consume(!a_consume->isChecked());
}

void qm_browser::set_xfade()
{
    if (mpd_connected)
    {
        if (a_xfade->isChecked())
            mpdCom->set_xfade(config->xfade_time);
        else
            mpdCom->set_xfade(0);
    }
    else
        set_menu_xfade(!a_xfade->isChecked());
}


void qm_browser::on_playlist_toggle()
{
    if (theSplitter->sizes().value(0) != 0)
    {
        plist_wmin_col2 = plist_view->columnWidth(2);
        plist_wmin_col3 = plist_view->columnWidth(3);
        plist_wmin_col5 = plist_view->columnWidth(5);
        
        splitter_left = theSplitter->sizes().value(0);
        splitter_right = theSplitter->sizes().value(1);
        
        QList<int> List;
        List.append(0);
        List.append(splitter_left + splitter_right);
        theSplitter->setSizes(List);
        
        // note: hide() does not add the handle width to viewport()->width()
        // don't just add 10 : The handle width can be overruled by the theme.
        int w = plist_view->viewport()->width() + theSplitter->handle(1)->width();
        left_group->hide();
        plist_view->set_maxpanel(true);
        bt_PLAYLIST->setIcon(QIcon(":/tr_expand2.png"));
        w = w - (plist_view->columnWidth(0) + plist_view->columnWidth(1) + plist_view->columnWidth(4));
        plist_view->setColumnWidth(2, w/3);
        plist_view->setColumnWidth(3, w/3);
        plist_view->setColumnWidth(5, w/3);
    }
    else
    {
        left_group->show();
        QList<int> List;
        List.append(splitter_left);
        List.append(splitter_right);
        theSplitter->setSizes(List);
        bt_PLAYLIST->setIcon(QIcon(":/tr_expand1.png"));
        plist_view->set_maxpanel(false);
        plist_view->setColumnWidth(2, plist_wmin_col2);
        plist_view->setColumnWidth(3, plist_wmin_col3);
        plist_view->setColumnWidth(5, plist_wmin_col5);
    }
}


void qm_browser::on_library_toggle()
{
    if (theSplitter->sizes().value(1) != 0)
    {
		db_mode_initial = config->browser_dbasemode;
		
        lib_wmin_col0 = lib_view->columnWidth(0);
        lib_wmin_col1 = lib_view->columnWidth(1);
        lib_wmin_col2 = lib_view->columnWidth(2);
        
        splitter_left = theSplitter->sizes().value(0);
        splitter_right = theSplitter->sizes().value(1);
        
        QList<int> List;
        List.append(splitter_left + splitter_right);
        List.append(0);
        theSplitter->setSizes(List);
        
        // note: hide() does not add the handle width to viewport()->width()
        // note: The handle-width set here can be overruled by the theme.
        int w = lib_view->viewport()->width() + theSplitter->handle(1)->width();
        right_group->hide();
        lib_view->set_maxpanel(true);
        bt_LIBRARY->setIcon(QIcon(":/tr_expand2.png"));
        
        lib_view->resizeColumnToContents(0);
        lib_view->resizeColumnToContents(1);
        lib_view->resizeColumnToContents(2);
        
        if (w/3 < lib_view->columnWidth(2))
            lib_view->setColumnWidth(2, w/3);
            
        w = w - lib_view->columnWidth(2);
		
        lib_view->setColumnWidth(1, w/2 - 30);
        lib_view->setColumnWidth(0, w/2 + 30);
    }
    else
    {
        right_group->show();
        lib_view->set_maxpanel(false);
        QList<int> List;
        List.append(splitter_left);
        List.append(splitter_right);
        theSplitter->setSizes(List);
        bt_LIBRARY->setIcon(QIcon(":/tr_expand1.png"));
		
		if (db_mode_initial == config->browser_dbasemode)
		{
			lib_view->setColumnWidth(0, lib_wmin_col0);
			lib_view->setColumnWidth(1, lib_wmin_col1);
			lib_view->setColumnWidth(2, lib_wmin_col2);
		}
		else
			lib_view->reload_root();
    }
}

// hack (splitter does't update its widget sizes when window resizes)
void qm_browser::resizeEvent ( QResizeEvent * rse)
{
    if (b_startup)
    {
        b_startup = false;
        return;
    }
    
    QList<int> List;
    int new_left = (rse->size().width() * theSplitter->sizes().value(0)) / rse->oldSize().width();
    List.append(new_left);
    int new_right =(rse->size().width() * theSplitter->sizes().value(1)) / rse->oldSize().width();
    List.append(new_right);
    theSplitter->setSizes(List);
    
    splittermoved();
    
    rse->accept();
}


void qm_browser::set_colors(QPalette pal, bool custom)
{
    lb_left_title->setPalette(pal);
    lb_right_title->setPalette(pal);
    
    if (custom)
    {
        lb_right_title->setFrameShape(QFrame::StyledPanel);
        lb_left_title->setFrameShape(QFrame::StyledPanel);
    }
    else
    {
        lb_right_title->setFrameShape(QFrame::NoFrame);
        lb_left_title->setFrameShape(QFrame::NoFrame);
    }
}


qm_browser::~qm_browser()
{}
