/*
*  qm_config.cpp
*  QUIMUP configuration class
*  (c) 2008-2010 Johan Spee
*
*  This file is part of Quimup
*
*  QUIMUP is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  QUIMUP is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program. If not, see http://www.gnu.org/licenses/.
*/

#include "qm_config.h"


qm_config::qm_config()
{
    load_config();
}


// Save the options in a config file
void qm_config::save_config()
{
    QSettings sts("coonsden", "quimup");
    // server related values
    sts.setValue("quitMPD_onquit",     quitMPD_onquit);
    sts.setValue("onquit_mpd_command", onquit_mpd_command);
    sts.setValue("startMPD_onstart",   startMPD_onstart);
    sts.setValue("onstart_mpd_command",onstart_mpd_command);
    sts.setValue("auto_connect",       auto_connect);
    
    sts.setValue("profile",            profile);
    
    sts.setValue("MPD_host_0",         MPD_host_0);
    sts.setValue("MPD_port_0",         MPD_port_0);
    sts.setValue("MPD_password_0",     MPD_password_0);
    sts.setValue("MPD_cofigpath_0",    MPD_cofigpath_0);
    
    sts.setValue("Profilename_1",      Profilename_1);
    sts.setValue("MPD_host_1",         MPD_host_1);
    sts.setValue("MPD_port_1",         MPD_port_1);
    sts.setValue("MPD_password_1",     MPD_password_1);
    sts.setValue("MPD_cofigpath_1",    MPD_cofigpath_1);
    
    sts.setValue("Profilename_2",      Profilename_2);
    sts.setValue("MPD_host_2",         MPD_host_2);
    sts.setValue("MPD_port_2",         MPD_port_2);
    sts.setValue("MPD_password_2",     MPD_password_2);
    sts.setValue("MPD_cofigpath_2",    MPD_cofigpath_2);
    
    sts.setValue("Profilename_3",      Profilename_3);
    sts.setValue("MPD_host_3",         MPD_host_3);
    sts.setValue("MPD_port_3",         MPD_port_3);
    sts.setValue("MPD_password_3",     MPD_password_3);
    sts.setValue("MPD_cofigpath_3",    MPD_cofigpath_3);
    
    sts.setValue("Profilename_4",      Profilename_4);
    sts.setValue("MPD_host_4",         MPD_host_4);
    sts.setValue("MPD_port_4",         MPD_port_4);
    sts.setValue("MPD_password_4",     MPD_password_4);
    sts.setValue("MPD_cofigpath_4",    MPD_cofigpath_4);
    
    sts.setValue("xfade_time",         xfade_time);
    
    // 'player' related values
    sts.setValue("scroller_delay",     scroller_delay);
    sts.setValue("playerWin_X",        playerWin_X);
    sts.setValue("playerWin_Y",        playerWin_Y);
    sts.setValue("playerWin_maxmode",  playerWin_maxmode);
    sts.setValue("use_trayicon",       use_trayicon);
    sts.setValue("use_timeleft",       use_timeleft);
    sts.setValue("always_ontop",       always_ontop);
    sts.setValue("tag_editor",         tag_editor);
    sts.setValue("art_viewer",         art_viewer);
    sts.setValue("file_manager",       file_manager);
    sts.setValue("rpgain_mode",        rpgain_mode);
    // 'browser window' values
    sts.setValue("browserWin_X",        browserWin_X);
    sts.setValue("browserWin_Y",        browserWin_Y);
    sts.setValue("browserWin_W",        browserWin_W);
    sts.setValue("browserWin_H",        browserWin_H);
    sts.setValue("browser_L_splitter",  browser_L_splitter);
    sts.setValue("browser_R_splitter",  browser_R_splitter);
    sts.setValue("browser_dbasemode",   browser_dbasemode);
    sts.setValue("browser_searchmode",  browser_searchmode);
    sts.setValue("browser_selectmode",  browser_selectmode);
    sts.setValue("sort_albums_year",  sort_albums_year);
    sts.setValue("ignore_leading_the",  ignore_leading_the);
    sts.setValue("disable_albumart",  disable_albumart);
    sts.setValue("search_string",   search_string);
    
    sts.setValue("auto_playfirst",   auto_playfirst);
    // 'settings window' values
    sts.setValue("settingsWin_X",      settingsWin_X);
    sts.setValue("settingsWin_Y",      settingsWin_Y);
    // font sizes
    sts.setValue("maintitle_fontsize", maintitle_fontsize);
    sts.setValue("codecinfo_fontsize", codecinfo_fontsize );
    sts.setValue("time_fontsize",      time_fontsize);
    sts.setValue("album_fontsize",     album_fontsize);
    sts.setValue("liblist_fontsize",   liblist_fontsize);
    sts.setValue("comment_fontsize",   comment_fontsize);
    sts.setValue("color_fg",           color_fg);
    sts.setValue("color_bg",           color_bg);
    sts.setValue("rb_sysdef_checked",  rb_sysdef_checked);
    sts.setValue("rb_qmcool_checked",  rb_qmcool_checked);
    sts.setValue("rb_custom_checked",  rb_custom_checked);
    sts.setValue("color_browser",      color_browser);
    
    // tooltips
    sts.setValue("show_tooltips",      show_tooltips);
    sts.sync();
    if (sts.status() == QSettings::NoError)
        printf ("Saving settings: OK\n");
    else
        printf ("Error saving settings\n");
}

// Get the options from the config file
void qm_config::load_config()
{
    QSettings sts("coonsden", "quimup");
    
    // server related values
    quitMPD_onquit    	= sts.value("quitMPD_onquit",     false  ).toBool();
    onquit_mpd_command  = sts.value("onquit_mpd_command", "mpd --kill").toString();
    startMPD_onstart    = sts.value("startMPD_onstart",   false  ).toBool();
    onstart_mpd_command = sts.value("onstart_mpd_command","mpd"  ).toString();
    auto_connect     	= sts.value("auto_connect",       true  ).toBool();
    
    profile    			= sts.value("profile",            0   ).toInt();
    
    MPD_host_0       	= sts.value("MPD_host_0",         "localhost" ).toString();
    MPD_port_0     		= sts.value("MPD_port_0",         6600  ).toInt();
    MPD_password_0   	= sts.value("MPD_password_0",     ""   ).toString();
    MPD_cofigpath_0   	= sts.value("MPD_cofigpath_0",    ""   ).toString();
    
    Profilename_1    	= sts.value("Profilename_1",    "Profile 1").toString();
    MPD_host_1       	= sts.value("MPD_host_1",         "localhost" ).toString();
    MPD_port_1     		= sts.value("MPD_port_1",         6600  ).toInt();
    MPD_password_1   	= sts.value("MPD_password_1",     ""   ).toString();
    MPD_cofigpath_1   	= sts.value("MPD_cofigpath_1",    ""   ).toString();
    
    Profilename_2    	= sts.value("Profilename_2",    "Profile 2").toString();
    MPD_host_2       	= sts.value("MPD_host_2",         "localhost" ).toString();
    MPD_port_2     		= sts.value("MPD_port_2",         6600  ).toInt();
    MPD_password_2   	= sts.value("MPD_password_2",     ""   ).toString();
    MPD_cofigpath_2  	= sts.value("MPD_cofigpath_2",    ""   ).toString();
    
    Profilename_3    	= sts.value("Profilename_3",    "Profile 3").toString();
    MPD_host_3       	= sts.value("MPD_host_3",         "localhost" ).toString();
    MPD_port_3     		= sts.value("MPD_port_3",         6600  ).toInt();
    MPD_password_3   	= sts.value("MPD_password_3",     ""   ).toString();
    MPD_cofigpath_3   	= sts.value("MPD_cofigpath_3",    ""   ).toString();
    
    Profilename_4    	= sts.value("Profilename_4",    "Profile 4").toString();
    MPD_host_4       	= sts.value("MPD_host_4",         "localhost" ).toString();
    MPD_port_4     		= sts.value("MPD_port_4",         6600  ).toInt();
    MPD_password_4   	= sts.value("MPD_password_4",     ""   ).toString();
    MPD_cofigpath_4   	= sts.value("MPD_cofigpath_4",    ""   ).toString();
    
    xfade_time   		= sts.value("xfade_time",          5   ).toInt();
    // 'player' related values
    scroller_delay    	= sts.value("scroller_delay",     60   ).toInt();
    playerWin_X     	= sts.value("playerWin_X",        300   ).toInt();
    if (playerWin_X < 0)
        playerWin_X = 0;
    playerWin_Y     	= sts.value("playerWin_Y",        200   ).toInt();
    if (playerWin_Y < 0)
        playerWin_Y = 0;
    playerWin_maxmode   = sts.value("playerWin_maxmode",  true  ).toBool();
    use_trayicon     	= sts.value("use_trayicon",       true  ).toBool();
    use_timeleft    	= sts.value("use_timeleft",       false  ).toBool();
    always_ontop    	= sts.value("always_ontop",       false  ).toBool();
    tag_editor    		= sts.value("tag_editor",         "easytag" ).toString();
    art_viewer    		= sts.value("art_viewer",         "gwenview" ).toString();
    file_manager    	= sts.value("file_manager",       "dolphin" ).toString();
    rpgain_mode    		= sts.value("rpgain_mode",        "album" ).toString();
    // 'browser window' values
    browserWin_X     	= sts.value("browserWin_X",         300   ).toInt();
    if (browserWin_X < 0)
        browserWin_X = 0;
    browserWin_Y     	= sts.value("browserWin_Y",         200   ).toInt();
    if (browserWin_Y < 0)
        browserWin_Y = 0;
    browserWin_W      	= sts.value("browserWin_W",         600   ).toInt();
    browserWin_H      	= sts.value("browserWin_H",         360   ).toInt();
    browser_L_splitter  = sts.value("browser_L_splitter",   260   ).toInt();
    browser_R_splitter  = sts.value("browser_R_splitter",   360   ).toInt();
    browser_dbasemode   = sts.value("browser_dbasemode",    0   ).toInt();
    browser_searchmode  = sts.value("browser_searchmode",   0   ).toInt();
    browser_selectmode  = sts.value("browser_selectmode",   0   ).toInt();
    sort_albums_year   	= sts.value("sort_albums_year",    false  ).toBool();
    ignore_leading_the  = sts.value("ignore_leading_the",  true  ).toBool();
    disable_albumart    = sts.value("disable_albumart",    false ).toBool();
    auto_playfirst   	= sts.value("auto_playfirst",      true ).toBool();
    search_string    	= sts.value("search_string",       "" ).toString();
    
    // 'settings window' values
    settingsWin_X  		= sts.value("settingsWin_X",            300   ).toInt();
    if (settingsWin_X < 0)
        settingsWin_X = 0;
    settingsWin_Y  = sts.value("settingsWin_Y",            200   ).toInt();
    if (settingsWin_Y < 0)
        settingsWin_Y = 0;
    // default font sizes
    maintitle_fontsize  = sts.value("maintitle_fontsize",  11   ).toInt();
    codecinfo_fontsize  = sts.value("codecinfo_fontsize",  9   ).toInt();
    time_fontsize     	= sts.value("time_fontsize",       9   ).toInt();
    album_fontsize     	= sts.value("album_fontsize",      9   ).toInt();
    liblist_fontsize    = sts.value("liblist_fontsize",    10   ).toInt();
    comment_fontsize    = sts.value("comment_fontsize",    8   ).toInt();
    color_fg      		= sts.value("color_fg",            "#304050" ).toString();
    color_bg      		= sts.value("color_bg",            "#F0F4F8" ).toString();
    rb_sysdef_checked 	= sts.value("rb_sysdef_checked",   false  ).toBool();
    rb_qmcool_checked 	= sts.value("rb_qmcool_checked",   true  ).toBool();
    rb_custom_checked 	= sts.value("rb_custom_checked",   false  ).toBool();
    color_browser  		= sts.value("color_browser",       true  ).toBool();
    // tooltips
    show_tooltips     	= sts.value("show_tooltips",       true  ).toBool();
    sts.sync();
    if (sts.status() == QSettings::NoError)
        printf ("Getting settings: OK\n");
    else
        printf ("Error getting settings. Using defaults.\n");
}

qm_config::~qm_config()
{}
