/*
*  qm_libraryview.h
*  QUIMUP library treeview
*  (c) 2008-2010 Johan Spee
*
*  This file is part of Quimup
*
*  QUIMUP is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  QUIMUP is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program. If not, see http://www.gnu.org/licenses/.
*/

#ifndef QM_LIBRARY_H
#define QM_LIBRARY_H

#include <QTreeWidget>
#include <QHeaderView>
#include <QTreeWidgetItem>
#include <QKeyEvent>
#include <QMenu>
#include <QFile>
#include <QMessageBox>
#include <QProcess>
#include <QScrollBar>
#include "qm_songlist.h"
#include "qm_mpdcom.h"
#include "qm_itemlist.h"
#include "qm_commandlist.h"
#include "qm_browser_ids.h"
#include "qm_config.h"

#define setdummy(bd)  setData(0, Qt::UserRole +  1, bd)
#define settype(tp)  setData(0, Qt::UserRole +  2, tp)
#define settime(tm)  setData(0, Qt::UserRole +  3, tm)
#define setartist(at)  setData(0, Qt::UserRole +  4, at)
#define setalbum(ab) setData(0, Qt::UserRole +  5, ab)
#define settitle(tt) setData(0, Qt::UserRole +  6, tt)
#define setfile(fl)  setData(0, Qt::UserRole +  8, fl)
#define settrack(tr)  setData(0, Qt::UserRole +  9, tr)
#define setname(nm)  setData(0, Qt::UserRole + 10, nm)
#define setgenre(gr)  setData(0, Qt::UserRole + 12, gr)

#define getdummy  data(0, Qt::UserRole +  1).toInt()
#define gettype   data(0, Qt::UserRole +  2).toInt()
#define gettime   data(0, Qt::UserRole +  3).toInt()
#define getartist   data(0, Qt::UserRole +  4).toString()
#define getalbum  data(0, Qt::UserRole +  5).toString()
#define gettitle  data(0, Qt::UserRole +  6).toString()
#define getfile   data(0, Qt::UserRole +  8).toString()
#define gettrack   data(0, Qt::UserRole +  9).toString()
#define getname   data(0, Qt::UserRole + 10).toString()
#define getgenre   data(0, Qt::UserRole + 12).toString()

class qm_libview : public QTreeWidget
{
	Q_OBJECT
	
public:
	qm_libview(QWidget * parent = 0);
	virtual ~qm_libview();
	
public slots:
	void set_font(QFont &);
	void set_mpdCom(qm_mpdCom*, bool);
	void set_config(qm_config*);
	void set_playlistModList(qm_commandList*);
	void reload_root();
	void on_plist_dropreceived(int);
	void set_maxpanel(bool);
	
private slots:
	void on_item_expanded(QTreeWidgetItem *);
	void on_SelectionChanged();
	void on_append();
	void on_createnew();
	void on_plistdrop();
	void on_delete();
	void on_opendir();
	void startDrag(Qt::DropActions);
	void on_mpd_plistsaved();
	
private:

	bool
		mpd_connected,
		b_dragdrop,
		b_panel_max;
	int plist_droppos,
		searched_mode_used;
	qm_mpdCom *mpdCom;
	qm_config *config;
	QMenu *ctxMenu;
	QAction *a_delete;
	QAction *a_opendir;
	
	qm_commandList *thePlaylistModList;
	void get_albums(QString searchfor = "*^*", bool sbd = false);
	void get_artists(QString searchfor = "*^*");
	void get_genres(QString searchfor = "*^*");
	void get_songs(QString searchfor = "*^*");
	void get_playlists();
	void get_folders();
	void get_albums_year();
	void expand_album(QTreeWidgetItem *);
	void expand_artist(QTreeWidgetItem *);
	void expand_playlist(QTreeWidgetItem *);
	void expand_folder(QTreeWidgetItem *);
	void expand_genre(QTreeWidgetItem *);
	void add_dummy_to(QTreeWidgetItem *);
	void take_dummy_from(QTreeWidgetItem *);
	void contextMenuEvent(QContextMenuEvent *);
	void add_songdata_toItem(QTreeWidgetItem *, qm_listitemInfo);
	void create_theModList(bool);
	void append_toModlist(QString, QString, QString);
	void append_toModlist_folder(QString);
	void append_toModlist_plist(QString);
	void set_local_features(bool);
	
signals:
	void sgnl_newmode(int);
	void sgnl_playlist_modification(bool);
	
};

#endif //  QM_LIBRARY_H
