/*
* qm_mpdcom.h
*  QUIMUP mpd communicator class
*  (c) 2008-2010 Johan Spee
*
*  This file is part of Quimup
*
*  QUIMUP is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  QUIMUP is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program. If not, see http://www.gnu.org/licenses/.
*/

#ifndef QM_MPDCOM_H
#define QM_MPDCOM_H


#include <mpd/client.h>
#include <stdlib.h>
#include <QObject>
#include <QTimer>
#include <QList>
#include <QListIterator>
#include <QTextStream>
#include <QFile>
#include <QDir>
#include <QDateTime>
#include <QMessageBox>
#include "qm_config.h"
#include "qm_songlist.h"
#include "qm_itemlist.h"
#include "qm_commandlist.h"

typedef struct qm_output
{
	int id;
	bool enabled;
	QString name;
}
qm_output;

typedef QList<qm_output> qm_outputList;
typedef QList<int> qm_IDlist;


class qm_mpdCom : public QObject
{
	Q_OBJECT
	
public:
	//  functions
	qm_mpdCom();
	virtual ~qm_mpdCom ();
	
public slots:
	bool mpd_connect();
	bool getReplayGainMode(QString);
	void prev();
	void stop();
	void play(bool reload = false);
	void pause();
	void resume();
	void next();
	void play_this(int);
	void set_random(bool);
	void set_repeat(bool);
	void set_single(bool);
	void set_consume(bool);
	void set_volume(int);
	void volume_up(int);
	void volume_down(int);
	void set_seek(int);
	void updatefile(QString);
	void update_all();
	QString get_version();
	qm_outputList getOutputs();
	qm_IDlist get_IDlist();
	void setOutputs(qm_outputList);
	int get_xfade();
	void set_xfade(int);
	void set_config(qm_config*);
	void configure();
	int get_listChanger();
	void execute_cmds(qm_commandList, bool);
	void clear_list();
	void shuffle_list();
	void get_statistics();
	void get_playlist();
	void reset_current_song();
	void show_messagebox(QString, QString);
	void setReplayGainMode(int);
	QString save_playlist(QString);
	QString save_selection(QStringList, QString);
	QString get_album_year(QString, QString);
	qm_itemList get_albumlist(bool);
	qm_itemList get_album_searchlist(QString);
	qm_itemList get_albums_for_artist(QString);
	qm_itemList get_artistlist();
	qm_itemList get_artist_searchlist(QString);
	qm_itemList get_playlistlist();
	qm_itemList get_folderlist(QString);
	qm_itemList get_songlist(QString, QString, QString);
	qm_itemList get_song_searchlist(QString);
	qm_itemList get_genrelist(QString);
	qm_itemList get_genre_artistlist(QString findgenre);
	qm_itemList get_playlist_itemlist(QString);
	qm_itemList get_yearlist();
	
signals:
	void sgnl_status(int, QString);
	void sgnl_statusInfo(mpd_status *);
	void sgnl_songInfo(qm_songInfo);
	void sgnl_connected(bool);
	void sgnl_plistupdate(qm_songList, int);
	void sgnl_dbase_statistics(QString);
	void sgnl_update_ready();
	void sgnl_plistsaved();
	
protected:

private slots:
	bool statusCheck();
	bool mpd_reconnect();
	
private:
	//  functions
	qm_songInfo get_songInfo_from(mpd_song *);
	QString get_musicPath();
	QString get_playlistPath();
	QString get_password();
	QString get_host();
	int get_port();
	QString get_string(QString);
	bool errorCheck(QString);
	QString fixTrackformat (QString);
	//  variables
	mpd_connection *conn;
	QTimer *statusLoop;
	QTimer *reconnectLoop;
	
	bool
	b_statCheckBusy,
	b_mpdconf_found,
	b_dbaseUpdating,
	b_connecting,
	b_reload,
	b_no_volume,
	autodetect;
	
	int
	current_playlist,
	current_songpos,
	current_songID,
	current_status,
	serverPort,
	current_volume,
	plist_lenght;
	
	QString
	serverName,
	serverPassword,
	mpdconf_path;
	
	qm_config *config; // pointer to the parent config
};

#endif // QM_MPDCOM_H
