/*
*  qm_player.cpp
*  QUIMUP main player window
*  (c) 2008-2010 Johan Spee
*
*  This file is part of Quimup
*
*  QUIMUP is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  QUIMUP is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program. If not, see http://www.gnu.org/licenses/.
*/

#include "qm_player.h"

qm_player::qm_player()
{
    if (objectName().isEmpty())
        setObjectName("player");
        
    setupUI();
    
    init_vars(); // incl. config
    
    retranslateUI();
    
    set_local_features(false);
    
    this->show();
    max_Win_W = this->width();
    max_Win_H = this->height();
    
    if (config->playerWin_maxmode)
    {
        windowH = max_Win_H;
        setFixedSize(max_Win_W, max_Win_H);
    }
    else
    { // minimize
        windowH = 110;
        center_widget->hide();
        setFixedSize(max_Win_W, windowH);
        config->playerWin_maxmode = false;
    }
    
    QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    setSizePolicy(sizePolicy);
    
    mpdCom->set_config(config);
    mpdCom->configure();
    
    // connect and set status
    if (config->auto_connect)
    {
        printf ("Autoconnecting, as configured\n");
        
        // if connection fails MPD is probably not running
        if (!mpdCom->mpd_connect())
        {
            if (config->startMPD_onstart)
            {
                printf ("Starting MPD, as configured\n");
                QProcess *proc;
                proc->startDetached(config->onstart_mpd_command);
                if ( !mpdCom->mpd_connect() )
                {
                    printf ("Failed to connect to mpd\n");
                    mpdCom->show_messagebox(trUtf8("Could not connect to mpd"), trUtf8("Check the 'connect' tab in the 'settings' window."));
                }
            }
            else
            {
                printf ("Failed to connect to mpd\n");
                mpdCom->show_messagebox(trUtf8("Could not connect to mpd"), trUtf8("Make sure mpd is running and check the 'connect' tab in the 'settings' window."));
            }
        }
    }
    else
    {
        printf ("NOT Connecting, as configured\n");
        on_connect_sgnl(false);
    }
}

// from mpdCom
void qm_player::on_connect_sgnl(bool isconnected)
{
    if (isconnected)
    {
        mpd_connected = true;
        settingsWin->set_mpdCom(mpdCom, true);
        browserWin->set_mpdCom(mpdCom, true);
        browserWin->plist_view->set_mpdCom(mpdCom, true);
        browserWin->lib_view->set_mpdCom(mpdCom, true);
        if (b_useTrayIcon)
            theTrayIcon->set_mpdCom(mpdCom, true);
        set_status(0, " ");
        set_local_features(config->mpd_local_features);
    }
    else
    {
        mpd_connected = false;
        settingsWin->set_mpdCom(mpdCom, false);
        browserWin->set_mpdCom(mpdCom, false);
        browserWin->plist_view->set_mpdCom(mpdCom, false);
        browserWin->lib_view->set_mpdCom(mpdCom, false);
        if (b_useTrayIcon)
            theTrayIcon->set_mpdCom(mpdCom, false);
        set_status(-1, trUtf8("Not connected") );
        set_local_features(false);
    }
}

void qm_player::setupUI()
{
    if (objectName().isEmpty())
        setObjectName(QString::fromUtf8("qm_player"));
        
    QPixmap qpx;
    qpx = QPixmap(":/mn_icon.png");
    setWindowIcon(qpx);
    setWindowTitle("Quimup");
    
    main_widget = new QWidget();
    vbox_all = new QVBoxLayout(main_widget);
    setCentralWidget(main_widget); // 'attatches' it to the window
    vbox_all->setMargin(6);
    vbox_all->setSpacing(6);
    
    Display = new QLabel(); // used as QFrame
    Display->setFixedSize(324,50);
    Display->setFrameShape(QFrame::StyledPanel);
    Display->setFrameShadow(QFrame::Sunken);
    
    lb_extension = new QLabel(Display);
    lb_extension->setGeometry(QRect(10, 28, 50, 20));
    lb_extension->setAlignment(Qt::AlignLeft|Qt::AlignVCenter);
    lb_extension->setWordWrap(false);
    
    lb_time = new clickLabel(Display);
    lb_time->setGeometry(QRect(200, 28, 114, 20));
    lb_time->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
    lb_time->setWordWrap(false);
    lb_time->setTextFormat(Qt::RichText);
    
    titleScroller = new qm_Scroller(Display);
    titleScroller->setGeometry(QRect(2, 2, 320, 24));
    titleScroller->setAlignment(Qt::AlignCenter|Qt::AlignVCenter);
    
    lb_kbps = new QLabel(Display);
    lb_kbps->setGeometry(QRect(66, 28, 64, 20));
    lb_kbps->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
    lb_kbps->setWordWrap(false);
    
    lb_kHz = new QLabel(Display);
    lb_kHz->setGeometry(QRect(140, 28, 64, 20));
    lb_kHz->setAlignment(Qt::AlignLeft|Qt::AlignVCenter);
    lb_kHz->setWordWrap(false);
    
    vbox_all->addWidget(Display);
    
    ds_progress = new qm_clickprogressbar();
	ds_progress->setOrientation (Qt::Horizontal);
    ds_progress->setFixedSize(324,8);
	ds_progress->setStyleSheet(
				"QSlider::groove:horizontal {"
				"margin: 0 0 0 0;"
				"border: 1px solid #aaa;"
				"background: transparent;"
				"height: 3px;"
				"border-radius: 2px;}"
				"QSlider::sub-page:horizontal {"
				"border-image: url(:/groove_bg_noconn.png); }");
	ds_progress->setMinimum(0);
    vbox_all->addWidget(ds_progress);
    
    spacer = new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Expanding);
    vbox_all->addItem(spacer);
    
    center_widget = new QWidget();
    hbox_center = new QHBoxLayout(center_widget);
    hbox_center->setMargin(0);
    hbox_center->setSpacing(6);
    hbox_center->setAlignment(Qt::AlignLeft|Qt::AlignTop);
    lb_aArt = new QLabel();
    lb_aArt->setFixedSize(200, 200);
    lb_aArt->setScaledContents(false);
    lb_aArt->setAlignment(Qt::AlignCenter|Qt::AlignVCenter);
    hbox_center->addWidget(lb_aArt);
    vbox_cright = new QVBoxLayout();
    vbox_cright->setMargin(0);
    vbox_cright->setSpacing(4);
    vbox_cright->setAlignment(Qt::AlignLeft|Qt::AlignTop);
    lb_album = new QLabel();
    lb_album->setWordWrap(true);
    lb_album->setMaximumSize(118, 170);
    lb_album->setMinimumSize(118, 0);
    lb_album->setTextFormat(Qt::RichText);
    lb_album->setAlignment(Qt::AlignLeft|Qt::AlignTop);
    vbox_cright->addWidget(lb_album);
    lb_comment = new QLabel();
    lb_comment->setWordWrap(true);
    lb_comment->setMaximumSize(118, 170);
    lb_comment->setTextFormat(Qt::RichText);
    lb_comment->setAlignment(Qt::AlignLeft|Qt::AlignTop);
    vbox_cright->addWidget(lb_comment);
    
    hbox_center->addLayout(vbox_cright);
    vbox_all->addWidget(center_widget);
    
    hbox_bottom = new QHBoxLayout();
    hbox_bottom->setMargin(0);
    hbox_bottom->setSpacing(4);
    bt_prev = new QPushButton();
    bt_prev->setFixedSize(34,28);
    bt_prev->setIcon(QIcon(":/mn_prev.png"));
    hbox_bottom->addWidget(bt_prev);
    bt_stop = new QPushButton();
    bt_stop->setFixedSize(34,28);
    bt_stop->setIcon(QIcon(":/mn_stop.png"));
    hbox_bottom->addWidget(bt_stop);
    bt_PlayPause = new QPushButton();
    bt_PlayPause->setFixedSize(34,28);
    bt_PlayPause->setIcon(QIcon(":/mn_play.png"));
    bt_PlayPause->setDefault(false);
    hbox_bottom->addWidget(bt_PlayPause);
    bt_next = new QPushButton();
    bt_next->setFixedSize(34,28);
    bt_next->setIcon(QIcon(":/mn_next.png"));
    hbox_bottom->addWidget(bt_next);
    bt_browser = new QPushButton();
    bt_browser->setFixedSize(34,28);
    bt_browser->setIcon(QIcon(":/mn_playlist.png"));
    hbox_bottom->addWidget(bt_browser);
    vol_slider = new QSlider(Qt::Horizontal);
    vol_slider->setRange(0,100);
    vol_slider->setFixedSize(58,28);
	vol_slider->setStyleSheet(
			"QSlider::groove:horizontal {"
			"margin: 1px 0 1px 0;"
			"border: 1px solid #aaa;"
			"background: transparent;"
			"height: 3px;"
			"border-radius: 2px; }"
            "QSlider::handle:horizontal {"
            "image: url(:/vol_handle.png);"
			"height: 15px;"
			"width: 6px;"
			"margin: -4px 0;}");
    hbox_bottom->addWidget(vol_slider);
    bt_sizer = new QPushButton();
    bt_sizer->setFixedSize(34,28);
    bt_sizer->setIcon(QIcon(":/mn_sizer.png"));
    hbox_bottom->addWidget(bt_sizer);
    bt_setts = new QPushButton();
    bt_setts->setFixedSize(34,28);
    bt_setts->setIcon(QIcon(":/mn_config.png"));
    hbox_bottom->addWidget(bt_setts);
    vbox_all->addLayout(hbox_bottom);
    
    // buttonmapper
    buttonMapper = new QSignalMapper();
    
    buttonMapper->setMapping( bt_PlayPause, ID_play );
    QObject::connect(bt_PlayPause, SIGNAL(clicked()), buttonMapper, SLOT(map()));
    
    buttonMapper->setMapping( bt_stop, ID_stop );
    QObject::connect(bt_stop, SIGNAL(clicked()), buttonMapper, SLOT(map()));
    
    buttonMapper->setMapping( bt_next, ID_next );
    QObject::connect(bt_next, SIGNAL(clicked()), buttonMapper, SLOT(map()));
    
    buttonMapper->setMapping( bt_prev, ID_prev );
    QObject::connect(bt_prev, SIGNAL(clicked()), buttonMapper, SLOT(map()));
    
    buttonMapper->setMapping( bt_browser, ID_plst );
    QObject::connect(bt_browser, SIGNAL(clicked()), buttonMapper, SLOT(map()));
    
    buttonMapper->setMapping( bt_setts, ID_sets );
    QObject::connect(bt_setts, SIGNAL(clicked()), buttonMapper, SLOT(map()));
    
    buttonMapper->setMapping(lb_time, ID_time );
    QObject::connect(lb_time, SIGNAL(clicked()), buttonMapper, SLOT(map()));
    
    buttonMapper->setMapping( bt_sizer, ID_mmax );
    QObject::connect(bt_sizer, SIGNAL(clicked()), buttonMapper, SLOT(map()));
    
    QObject::connect( buttonMapper, SIGNAL( mapped(int) ), this, SLOT( on_Signal(int) ) );
    // end buttonmapper
    
    QObject::connect(ds_progress, SIGNAL(pressed(QMouseEvent*)), this, SLOT(LockProgress(QMouseEvent*)));
    QObject::connect(ds_progress, SIGNAL(clicked()), this, SLOT(unLockProgress()));
    
    minimizer = new QTimer();
    QObject::connect(minimizer, SIGNAL(timeout()), this, SLOT(miniMice()));
    maximizer = new QTimer();
    QObject::connect(maximizer, SIGNAL(timeout()), this, SLOT(maxiMice()));
    
    this->setContextMenuPolicy(Qt::CustomContextMenu);
    QObject::connect(this, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(on_contextmenu()) );
    
    // CONTEXT MENU
    ctxMenu = new QMenu();
    
    QAction *title = new QAction(ctxMenu);
    title->setText(trUtf8("Folder operations:"));
    title->setEnabled(false);
    ctxMenu->addAction(title);
    
    ctxMenu->addSeparator();
    
    openaart = new QAction(ctxMenu);
    openaart->setText(trUtf8("Browse albumart"));
    openaart->setIcon(QIcon(":/menu_view.png"));
    QObject::connect( openaart, SIGNAL( triggered() ), this, SLOT( showAlbumArt() ) );
    ctxMenu->addAction(openaart);
    
    edittags = new QAction(ctxMenu);
    edittags->setText(trUtf8("Edit audio tags"));
    edittags->setIcon(QIcon(":/menu_edit.png"));
    QObject::connect( edittags, SIGNAL( triggered() ), this, SLOT( editTags() ) );
    ctxMenu->addAction(edittags);
    
    loadtags = new QAction(ctxMenu);
    loadtags->setText(trUtf8("Reload audio tags"));
    loadtags->setIcon(QIcon(":/menu_load.png"));
    QObject::connect( loadtags, SIGNAL( triggered() ), this, SLOT( reloadTags() ) );
    ctxMenu->addAction(loadtags);
    
    opendir = new QAction(ctxMenu);
    opendir->setText(trUtf8("Open in file manager"));
    opendir->setIcon(QIcon(":/menu_dir.png"));
    QObject::connect( opendir, SIGNAL( triggered() ), this, SLOT( openDirectory() ) );
    ctxMenu->addAction(opendir);
    
    QObject::connect(vol_slider, SIGNAL(valueChanged(int)), this, SLOT(on_volumedial(int)));
}


void qm_player::on_volumedial(int vol)
{

    if (b_mpdSetsVolume || !mpd_connected)
        return;
        
    if (vol != current_volume)
    {
        current_volume = vol;
        mpdCom->set_volume(vol);
		if (config->show_tooltips)
			vol_slider->setToolTip("Volume " + QString::number(vol) + " %");
    }
}

void qm_player::miniMice()
{
    // fade the album art first
    if (b_minmax_xfade)
    {
        alphacycler -= 6;
        if (alphacycler < 0)
            alphacycler = 0;
            
        int a = alphacycler;
        QImage xfade = img_xfade_input;
        for ( int j = 0; j < aArt_H; j++ )
        {
            QRgb *px = (QRgb*)xfade.scanLine(j);
            for ( int k = 0; k < aArt_W; k++ )
            {
                uchar r = uchar( ( int(qRed(*px))   * a  +  cR * (255 - a)) / 255 );
                uchar g = uchar( ( int(qGreen(*px)) * a  +  cG * (255 - a)) / 255 );
                uchar b = uchar( ( int(qBlue(*px))  * a  +  cB * (255 - a)) / 255 );
                *px = qRgba( r, g, b, 255);
                px++;
            }
        }
        
        pxb_xfading = QPixmap::fromImage(xfade);
        
        lb_aArt->setPixmap(pxb_xfading);
        
        if (alphacycler == 0)
        {
            center_widget->hide();
            b_minmax_busy = false;
            b_minmax_xfade = false;
        }
        return;
    }
    
    windowH -= 16;
    if (windowH >= 110)
    {
        setFixedSize(max_Win_W, windowH);
    }
    else
    {
        setFixedSize(max_Win_W, 110);
        minimizer->stop();
    }
}


void qm_player::init_vars()
{
    browserWin = new qm_browser();
    QObject::connect(browserWin->plist_view, SIGNAL(sgnl_curentsong_deleted()), this, SLOT(on_curentsong_deleted()));
    
    settingsWin = new qm_settings();
    QObject::connect(settingsWin, SIGNAL(sgnl_fonts()), this, SLOT(set_fonts()));
    QObject::connect(settingsWin, SIGNAL(sgnl_colors()), this, SLOT(set_colors()));
    QObject::connect(settingsWin, SIGNAL(sgnl_ontopmode()), this, SLOT(set_ontop()));
    
    mpdCom = new qm_mpdCom();
    QObject::connect( mpdCom, SIGNAL( sgnl_songInfo(qm_songInfo)), this, SLOT( on_newSong(qm_songInfo)));
    QObject::connect( mpdCom, SIGNAL( sgnl_statusInfo(mpd_status *)), this, SLOT( on_newStatus(mpd_status *)));
    QObject::connect( mpdCom, SIGNAL( sgnl_connected (bool)), this, SLOT( on_connect_sgnl(bool)));
    QObject::connect( mpdCom, SIGNAL( sgnl_status(int, QString)), this, SLOT( set_status(int, QString)));
    
    config = new qm_config(); // constructor loads values
    settingsWin->set_config(config);
    browserWin->set_config(config);
    browserWin->plist_view->set_config(config);
    browserWin->lib_view->set_config(config);
    
    current_status = -99; // allow all
    current_volume = 0;
	if (config->show_tooltips)
		vol_slider->setToolTip("Volume 0 %");
    songTotalTime = 0;
	songPrevTime = 0;
    b_repeat = false;
    b_random = false;
    b_single = false;
    b_consume = false;
    b_xfade = false;
    mpd_connected = false;
    b_mpdSetsVolume = false;
    b_stream = false;
    b_minmax_busy = false;
    b_settingsWin_hidden = false;
    b_browserWin_hidden = false;
    b_nosong = false;
    b_lockProgress = false;
    b_plReallyClose = false;
    b_need_newSong = false;
    b_reshow_browser = false;
    b_reshow_settings = false;
    current_art_path = "x";
    
    if (config->scroller_delay < 20)
        config->scroller_delay = 20;
    titleScroller->set_delay( config->scroller_delay );
    
	// 'load' or they will disappear after a repaint
    img_mn_play = QIcon(":/mn_play");
    img_mn_pause = QIcon(":/mn_pause");
    pxb_led_playing.load(":/led_playing");
    pxb_led_noconn.load(":/led_noconn");
    pxb_led_paused.load(":/led_paused");
    pxb_led_stopped.load(":/led_stopped");
    pxb_led_idle.load(":/led_idle");
    
    // APPLY THE DEFAULT BACKGROUND COLORS TO 'NOPIC' & 'ONAIR'
    // transparency is filled with this color to allow xfading
    QPalette pal (this->palette()); // grab any non-colorized object
    QColor col = pal.color(QPalette::Window);
    cR = col.red();
    cG = col.green();
    cB = col.blue();
    
    img_nopic.load(":/nopic");
    {  // fill transparency with background color (for fading)
        for ( int j = 0; j < 200; j++ )
        {
            QRgb *px = (QRgb*)img_nopic.scanLine(j);
            for ( int k = 0; k < 200; k++ )
            {
                int a = qAlpha(*px);
                uchar r = uchar( ( int(qRed(*px))   * a  +  cR * (255 - a)) / 255 );
                uchar g = uchar( ( int(qGreen(*px)) * a  +  cG * (255 - a)) / 255 );
                uchar b = uchar( ( int(qBlue(*px))  * a  +  cB * (255 - a)) / 255 );
                *px = qRgba( r, g, b, 255);
                px++;
            }
        }
    }
    img_onair.load(":/onair");
    {  // fill transparency with background color (for fading)
        for ( int j = 0; j < 200; j++ )
        {
            QRgb *px = (QRgb*)img_onair.scanLine(j);
            for ( int k = 0; k < 200; k++ )
            {
                int a = qAlpha(*px);
                uchar r = uchar( ( int(qRed(*px))   * a  +  cR * (255 - a)) / 255 );
                uchar g = uchar( ( int(qGreen(*px)) * a  +  cG * (255 - a)) / 255 );
                uchar b = uchar( ( int(qBlue(*px))  * a  +  cB * (255 - a)) / 255 );
                *px = qRgba( r, g, b, 255);
                px++;
            }
        }
    }
    // << APPLY THE DEFAULT BACKGROUND COLORS
    
    set_albumart("nopic");
    
    if (config->use_trayicon)
    {
        theTrayIcon = new qm_trayIcon(this);
        theTrayIcon->show();
        QObject::connect(theTrayIcon, SIGNAL(clicked()), this, SLOT(on_trayClick()));
        QObject::connect(theTrayIcon, SIGNAL(traySignal_menu(int)), this, SLOT(on_Signal(int)));
        theTrayIcon->set_mode(ID_idle);
        theTrayIcon->setTip(" Idle ");
        b_useTrayIcon = true;
    }
    else
        b_useTrayIcon = false;
        
    set_colors();
    set_fonts();
    
    move(config->playerWin_X, config->playerWin_Y);
	
	vol_slider->setFocus(Qt::OtherFocusReason);
	
    set_ontop();
}


void qm_player::maxiMice()
{
    // fade the album art last
    if (b_minmax_xfade)
    {
        alphacycler += 6;
        if (alphacycler >= 255)
        {
            b_minmax_busy = false;
            lb_aArt->setPixmap(pxb_albmart);
            maximizer->stop();
            return ; // done
        }
        
        int a = alphacycler;
        QImage xfade = img_xfade_input;
        for ( int j = 0; j < aArt_H; j++ )
        {
            QRgb *px = (QRgb*)xfade.scanLine(j);
            for ( int k = 0; k < aArt_W; k++ )
            {
                uchar r = uchar( ( int(qRed(*px))   * a  +  cR * (255 - a)) / 255 );
                uchar g = uchar( ( int(qGreen(*px)) * a  +  cG * (255 - a)) / 255 );
                uchar b = uchar( ( int(qBlue(*px))  * a  +  cB * (255 - a)) / 255 );
                *px = qRgba( r, g, b, 255);
                px++;
            }
        }
        
        pxb_xfading = QPixmap::fromImage(xfade);
        lb_aArt->setPixmap(pxb_xfading);
        return;
    }
    
    windowH += 16;
    if (windowH <= max_Win_H)
    {
        setFixedSize(max_Win_W, windowH);
    }
    else
    {
        windowH = max_Win_H;
        setFixedSize(max_Win_W, max_Win_H);
        center_widget->show();
        b_minmax_xfade = true;
    }
}


void qm_player::on_trayClick()
{
    if (isVisible())
    {
        hide();
    }
    else
    {
        move(config->playerWin_X, config->playerWin_Y);
        showNormal();
    }
}

void qm_player::on_curentsong_deleted()
{
    // setup a "no song" situation and let mpd take it from there.
    qm_songInfo newSong;
    newSong.nosong = true;
    on_newSong(newSong);
    set_status(MPD_STATE_UNKNOWN); // also sets the plist_view
}

void qm_player::on_newSong(qm_songInfo newSong)
{
    if (newSong.nosong)
    {
        if (b_nosong)
            return;
        // else
        b_nosong = true;
        browserWin->plist_view->set_newsong(-1);
        return;
    }
    else
        browserWin->plist_view->set_newsong(newSong.songNr);
        
    // in "idle" state we didn't set the title etc.
    if (current_status == MPD_STATE_UNKNOWN)
    {
        b_need_newSong = true;
        return;
    }
    else
        b_need_newSong = false;
        
    printf ("New song: %i\n", newSong.songNr);
    b_nosong = false;
    
    QString fname = "";
    QString path = "";
    
    if (!newSong.file.isEmpty())
    {
        fname = newSong.file;
        // current dir, from mpd's music dir, exc the last "/"
        current_mpd_dir = fname.left(fname.lastIndexOf("/"));
        // path, from mpd's music dir, inc the last "/"
        path = fname.left(fname.lastIndexOf("/") + 1);
        // remove path, keep filename only
        int i = fname.lastIndexOf( '/' ) + 1;
        fname = fname.right(fname.length() - i);
    }
    
    // streams
    if (path.startsWith("http:"))
    {
        current_mpd_dir = "";
        set_albumart(path);
        ds_progress->setValue(0);
        lb_extension->setText("URL");
        lb_album->setText(trUtf8("Audio stream"));
        // note: newSong->artist is NULL !
        QString artist_title = trUtf8("no info");
        if (!newSong.title.isEmpty())
        {
            artist_title = newSong.title;
        }
        else
        {
            if (!newSong.name.isEmpty())
                artist_title = newSong.name;
        }
        titleScroller->set_title(artist_title);
        
        if (b_useTrayIcon)
            theTrayIcon->setTip(artist_title);
        b_stream = true;
        return;
    }
    else
        b_stream = false;
        
    if (!config->mpd_local_features || config->disable_albumart)
        set_albumart("nopic");
    else
        set_albumart(path);
        
    // extension
    if (fname.isEmpty())
    {
        lb_extension->setText("");
    }
    else
    {
        QString aString = fname;
        int i = aString.lastIndexOf( '.' ) + 1;
        aString = aString.right(aString.length() - i);
        aString = aString.toUpper();
        lb_extension->setText(aString);
    }
    
    if (newSong.time)
    {
        songTotalTime = newSong.time;
		ds_progress->setRange(0,songTotalTime);
    }
    
    // artist
    QString artist = "?";
    if (!newSong.artist.isEmpty())
        artist = newSong.artist;
        
    // title
    QString title = "?";
    if (!newSong.title.isEmpty())
        title = newSong.title;
        
        
    if (newSong.artist.isEmpty() && newSong.title.isEmpty())
    {
        if (fname.isEmpty())
        {
            titleScroller->set_title("[?]");
            if (b_useTrayIcon)
                theTrayIcon->setTip(trUtf8("&nbsp;No Info&nbsp;"));
        }
        else
        {
            titleScroller->set_title(fname);
            if (b_useTrayIcon)
                theTrayIcon->setTip("&nbsp;" + fname + "&nbsp;");
        }
    }
    else
    {
        titleScroller->set_title(artist, title);
        if (b_useTrayIcon)
        {
            QString artist_tip = "&nbsp;<b>" + artist + "</b>&nbsp;";
            QString title_tip = "&nbsp;<i>" + title + "&nbsp;</i>";
            theTrayIcon->setTip( artist_tip + "<br>" + title_tip);
        }
    }
    
    //album
    QString album = "<b>?</b>";
    if (!newSong.album.isEmpty())
        album = "<b>" + newSong.album + "</b>";
    // year
    if (!newSong.date.isEmpty())
        album.append("<br>" + newSong.date);
        
    lb_album->setText(album);
    
    // comment
    lb_comment->setFixedSize(118, 180 - lb_album->height());
    lb_comment->setText(newSong.comment);
    
}


void qm_player::on_newStatus(mpd_status *sInfo)
{
    // set the playback modes in the playlist menu
    if ( mpd_status_get_repeat(sInfo) != b_repeat)
    {
        browserWin->set_menu_repeat(mpd_status_get_repeat(sInfo));
        b_repeat = mpd_status_get_repeat(sInfo);
    }
    
    if (mpd_status_get_random(sInfo) != b_random)
    {
        browserWin->set_menu_random(mpd_status_get_random(sInfo));
        b_random = mpd_status_get_random(sInfo);
    }
    
    if (mpd_status_get_single(sInfo) != b_single)
    {
        browserWin->set_menu_single(mpd_status_get_single(sInfo));
        b_single = mpd_status_get_single(sInfo);
    }
    
    if (mpd_status_get_consume(sInfo) != b_consume)
    {
        browserWin->set_menu_consume(mpd_status_get_consume(sInfo));
        b_consume = mpd_status_get_consume(sInfo);
    }
    
    if (mpd_status_get_crossfade(sInfo) > 0 && !b_xfade)
    {
        b_xfade = true;
        browserWin->set_menu_xfade(b_xfade);
    }
    
    if (mpd_status_get_crossfade(sInfo) == 0 && b_xfade)
    {
        b_xfade = false;
        browserWin->set_menu_xfade(b_xfade);
    }
    
    // if status has changed...
    if (b_nosong && current_status != MPD_STATE_UNKNOWN)
        set_status(MPD_STATE_UNKNOWN);
    else
        if (mpd_status_get_state(sInfo) != current_status)
            set_status(mpd_status_get_state(sInfo));
            
    // set the other status info
    if (mpd_status_get_state(sInfo) != MPD_STATE_UNKNOWN && current_status != MPD_STATE_UNKNOWN)
    {
        int timenow = mpd_status_get_elapsed_time(sInfo);
        
        if (!b_stream && !b_lockProgress && songTotalTime > 0)
        {
			if (timenow != songPrevTime)
			{
				songPrevTime = timenow;
				// some tracks play a bit too long (honestly)
				if (timenow > songTotalTime)
					timenow = songTotalTime;
				ds_progress->setValue(timenow);
			}
        }
        
        if (b_stream || songTotalTime == 0)
        {
            lb_time->setText("[&#187;]");
        }
        else
        {
            if (config->use_timeleft)
                timenow = songTotalTime - timenow;
                
            QString nowtime = "";
            if (config->use_timeleft)
                nowtime += "-";
            nowtime += into_time((int)timenow);
            lb_time->setText(nowtime + " [" + into_time((int)songTotalTime) + "]");
        }
        
        if (mpd_status_get_kbit_rate(sInfo) > 0)
            lb_kbps->setText(QString::number(mpd_status_get_kbit_rate(sInfo)) + " kb/s ");
        else
            lb_kbps->setText("");
            
        if (mpd_status_get_audio_format(sInfo)) // can return NULL, esp. with streams!
        {
            double dbl_khz = (double)mpd_status_get_audio_format(sInfo)->sample_rate / 1000;
            lb_kHz->setText(QString::number(dbl_khz) + " kHz");
        }
        else
            lb_kHz->setText("");
    }
    
    if (current_volume != mpd_status_get_volume(sInfo))
    {
        b_mpdSetsVolume = true;
        /* slider Scale is set 0 to 100
        	mpd also reports 0 to 100 */
        current_volume = mpd_status_get_volume(sInfo);
        vol_slider->setValue(current_volume);
		if (config->show_tooltips)
			vol_slider->setToolTip("Volume " + QString::number(current_volume) + " %");
        b_mpdSetsVolume = false;
    }
}

void qm_player::retranslateUI()
{
    if (config->show_tooltips)
    {
        lb_time->setToolTip(trUtf8(" Click to toggle time mode "));
        bt_sizer->setToolTip(trUtf8(" Mini-Maxi mode "));
        bt_browser->setToolTip(trUtf8(" Browser (Library & Playlist) "));
        bt_setts->setToolTip(trUtf8(" Settings "));
    }
}

// Signal handler.
void qm_player::on_Signal(int sigID)
{
    switch (sigID)
    {
    case ID_prev:
        {
            if (current_status != MPD_STATE_PLAY || !mpd_connected)
                break;
            mpdCom->prev();
            break;
        }
        
    case ID_stop:
        {
            if (current_status == MPD_STATE_UNKNOWN ||
                    current_status == MPD_STATE_STOP || !mpd_connected)
                break;
                
            set_status(MPD_STATE_STOP, "");
            titleScroller->pause(true);
            mpdCom->stop();
            break;
        }
        
    case ID_play:
        {
            if (!mpd_connected)
                break;
                
            switch (current_status)
            {
            case MPD_STATE_PLAY:
                {
                    set_status(MPD_STATE_PAUSE, "");
                    mpdCom->pause();
                    break;
                }
                
            case MPD_STATE_PAUSE:
                {
                    set_status(MPD_STATE_PLAY, "");
                    mpdCom->resume();
                    break;
                }
                
            case MPD_STATE_STOP:
                {
                    set_status(MPD_STATE_PLAY, "");
                    mpdCom->play();
                    break;
                }
                
            case MPD_STATE_UNKNOWN:
                {
                    set_status(MPD_STATE_PLAY, "");
                    mpdCom->play();
                    break;
                }
                
            default:
                {
                    break;
                }
            }
            break;
        }
        
    case ID_next:
        {
            if (current_status != MPD_STATE_PLAY || !mpd_connected)
                break;
            mpdCom->next();
            break;
        }
        
    case ID_sets:
        {
            if (settingsWin->isVisible())
                settingsWin->hide();
            else
            {
                settingsWin->set_config(config); // reset unsaved changes
                settingsWin->show();
                settingsWin->setWindowState(Qt::WindowNoState);
            }
            break;
        }
        
    case ID_plst:
        {
            if (browserWin->isVisible())
                browserWin->hide();
            else
            {
                browserWin->show();
                browserWin->setWindowState(Qt::WindowNoState);
            }
            break;
        }
        
    case ID_mmax:
        {
            if (b_minmax_busy)
                break;
                
            if (config->playerWin_maxmode)
            {
                img_xfade_input = pxb_albmart.toImage();
                b_minmax_xfade = true;
                b_minmax_busy = true;
                alphacycler = 255;
                minimizer->start(12);
                config->playerWin_maxmode = false;
            }
            else
            {
                img_xfade_input = pxb_albmart.toImage();
                b_minmax_xfade = false;
                b_minmax_busy = true;
                alphacycler = 0;
                maximizer->start(12);
                config->playerWin_maxmode = true;
            }
            break;
        }
        
    case ID_time:
        {
            config->use_timeleft = !config->use_timeleft;
            break;
        }
        
    case ID_exit:
        {
            b_plReallyClose = true;
            close();
            break;
        }
        
    default:
        {
            break;
        }
    }
	// focus goes back to the volume slider
	vol_slider->setFocus(Qt::OtherFocusReason);
}



void qm_player::set_status(int status, QString message)
{
    if (status == current_status)
        return;
        
    // in "idle" mode only respond to play an not connected
    if ( current_status == MPD_STATE_UNKNOWN && !(status == MPD_STATE_PLAY || status == -1) )
        return;
        
    if (b_need_newSong)
        mpdCom->reset_current_song();
        
    browserWin->plist_view->set_status(status);
    
    if (!message.isEmpty())
        titleScroller->set_title(message);
        
    switch (status)
    {
    case -1: // not connected
        {
            bt_PlayPause->setIcon(img_mn_play);
            lb_time->setText("");
            lb_kHz->setText("");
            lb_extension->setText("");
            lb_kbps->setText("");
            lb_album->setText("");
            lb_comment->setText("");
            set_albumart("nopic");
            ds_progress->setStyleSheet(
				"QSlider::groove:horizontal {"
				"margin: 0 0 0 0;"
				"border: 1px solid #aaa;"
				"background-image: url(:/groove_bg_noconn.png);"
				"height: 3px;"
				"border-radius: 2px; }");
			ds_progress->setValue(0);
            if (b_useTrayIcon)
            {
                theTrayIcon->setTip(trUtf8(" Not Connected "));
                theTrayIcon->set_mode(ID_nocn);
            }
            break;
        }
        
    case MPD_STATE_STOP: // 1
        {
            bt_PlayPause->setIcon(img_mn_play);
            titleScroller->pause(true);
            ds_progress->setStyleSheet(
				"QSlider::groove:horizontal {"
				"margin: 0 0 0 0;"
				"border: 1px solid #aaa;"
				"background-image: url(:/groove_bg_conn.png);"
				"height: 3px;"
				"border-radius: 2px; }"); 
			ds_progress->setValue(0); 
            if (b_useTrayIcon)
                theTrayIcon->set_mode(ID_stop);
            break;
        }
        
    case MPD_STATE_PLAY: // 2
        {
            bt_PlayPause->setIcon(img_mn_pause);
            titleScroller->pause(false);
            ds_progress->setStyleSheet(
				"QSlider::groove:horizontal {"
				"margin: 0 0 0 0;"
				"border: 1px solid #aaa;"
				"background-image: url(:/groove_bg_conn.png);"
				"height: 3px;"
				"border-radius: 2px;}"
				"QSlider::sub-page:horizontal {"
				"border-image: url(:/groove_sub_play.png); }"); 
			if (b_useTrayIcon)
                theTrayIcon->set_mode(ID_play);
            break;
        }
        
    case MPD_STATE_PAUSE: // 3
        {
            bt_PlayPause->setIcon(img_mn_play);
            titleScroller->pause(true);
            ds_progress->setStyleSheet(
				"QSlider::groove:horizontal {"
				"margin: 0 0 0 0;"
				"border: 1px solid #aaa;"
				"background-image: url(:/groove_bg_conn.png);"
				"height: 3px;"
				"border-radius: 2px;}"
				"QSlider::sub-page:horizontal {"
				"border-image: url(:/groove_sub_play.png); }"); 
            if (b_useTrayIcon)
                theTrayIcon->set_mode(ID_paus);
            break;
        }
        
    default: // 0 (MPD_STATE_UNKNOWN) "Idle"
        {
            titleScroller->set_title("&nbsp;");
            bt_PlayPause->setIcon(img_mn_play);
            lb_time->setText("");
            lb_kHz->setText("");
            lb_extension->setText("");
            lb_kbps->setText("");
            lb_album->setText("");
            lb_comment->setText("");
            set_albumart("nopic");
            ds_progress->setStyleSheet(
				"QSlider::groove:horizontal {"
				"margin: 0 0 0 0;"
				"border: 1px solid #aaa;"
				"background-image: url(:/groove_bg_conn.png);"
				"height: 3px;"
				"border-radius: 2px; }");
			ds_progress->setValue(0);
            if (b_useTrayIcon)
            {
                theTrayIcon->set_mode(ID_idle);
                theTrayIcon->setTip(trUtf8(" Idle "));
            }
            break;
        }
    }
    current_status = status;
}


QString qm_player::into_time(int time)
{
    QString formattedTime = "";
    int mins = (int)(time / 60);
    formattedTime += QString::number((mins)) + ":";
    int secs = time % 60;
    if (secs < 10)
        formattedTime += "0";
    formattedTime += QString::number((secs));
    return formattedTime;
}


void qm_player::set_colors()
{
    QPalette pal ;
    // get R G B to fill transparency in (album art) images
    QColor background = pal.color(QPalette::Window);
    cR = background.red();
    cG = background.green();
    cB = background.blue();
    
    QString foregdcol;
    QString backgdcol;
    
    if (config->rb_qmcool_checked)
    {
        foregdcol = "#1E3C69";
        backgdcol = "#D3E4F0";
    }
    else
        if (config->rb_custom_checked)
        {
            foregdcol = config->color_fg;
            backgdcol = config->color_bg;
        }
        
    if (!config->rb_sysdef_checked)
    {
        pal.setColor(QPalette::Window, QColor(backgdcol)); // background
        pal.setColor(QPalette::Foreground, QColor(foregdcol)); // text
    }
    else
    {  // system default, but 'midlight' as background for the main display
        pal.setColor(QPalette::Window, QColor(pal.color(QPalette::Midlight)));
    }
    
    if (config->color_browser)
        browserWin->set_colors(pal, true);
    else
        browserWin->set_colors(this->palette(), false);
        
    Display->setPalette(pal);
    Display->setAutoFillBackground(true); // siblings are transparent by default
    lb_kbps->setPalette(pal);
    lb_kbps->setAutoFillBackground(true);
    lb_time->setPalette(pal);
    lb_time->setAutoFillBackground(true);
    titleScroller->setPalette(pal);
    titleScroller->setAutoFillBackground(true);
    lb_kHz->setPalette(pal);
    lb_kHz->setAutoFillBackground(true);
    lb_extension->setPalette(pal);
    lb_extension->setAutoFillBackground(true);  
}

//  Mouse pressed on progress slider
void qm_player::LockProgress(QMouseEvent *e)
{
    if (!b_stream && current_status == MPD_STATE_PLAY && mpd_connected)
    {
        b_lockProgress = TRUE;
        // Get mouse x pos within widget
        int x = e->x();
        x = (x * songTotalTime) / 324; // 324 is width of widget
        ds_progress->setValue(x);
    }
}

// Mouse released on progress slider.
void qm_player::unLockProgress()
{
    if (!b_stream && current_status == MPD_STATE_PLAY && mpd_connected)
    {
        mpdCom->set_seek( ds_progress->value() );
        b_lockProgress = FALSE;
    }
}


void qm_player::set_fonts()
{
    QFont font;
    
    font.setPointSize(config->maintitle_fontsize);
    titleScroller->setFont(font);
    
    font.setPointSize(config->codecinfo_fontsize);
    lb_extension->setFont (font);
    lb_kHz->setFont (font);
    lb_kbps->setFont (font);
    
    QLabel testlabel;
    testlabel.setFont (font);
    testlabel.setText("1024 kb/s");
    int w_kbps = (testlabel.minimumSizeHint()).width ();
    
    bool ok_to_show_khz = true;
    // hide kHz label when kb/s needs the space
    if (w_kbps > 64)
    {
        lb_kHz->hide();
        ok_to_show_khz = false;
        lb_kbps->setGeometry(66, 28, w_kbps, 20);
    }
    else
    {
        lb_kHz->show();
        lb_kbps->setGeometry(66, 28, 64, 20);
    }
    
    
    font.setPointSize(config->time_fontsize);
    lb_time->setFont (font);
    
    testlabel.setFont (font);
    testlabel.setText("-00:00 [00:00]");
    int w_time = (testlabel.minimumSizeHint()).width ();
    
    // hide kHz label when time needs the space
    if (w_time > 114)
    {
        lb_kHz->hide();
        lb_time->setGeometry(QRect(200 - (w_time - 114), 28, w_time, 20));
    }
    else
    {
        if (ok_to_show_khz)
            lb_kHz->show();
        lb_time->setGeometry(QRect(200, 28, 114, 20));
    }
    
    font.setPointSize(config->album_fontsize);
    lb_album->setFont (font);
    
    font.setPointSize(config->comment_fontsize);
    lb_comment->setFont (font);
    
    font.setPointSize(config->liblist_fontsize);
    browserWin->set_font(font);
}


void qm_player::set_albumart(QString sub_path)
{
    QImage temp;
    bool b_skip_load = false;
    bool b_file_ok = false;
    
    if (sub_path.isEmpty() || sub_path == "nopic")
    {
        if (current_art_path == "nopic")
            return;
        temp = QImage(img_nopic);
        current_art_path = "nopic";
        b_skip_load = true;
        b_file_ok = true;
    }
    
    if (sub_path.startsWith("http:"))
    {
        if (current_art_path == "onair")
            return;
        temp = QImage(img_onair);
        current_art_path = "onair";
        b_skip_load = true;
        b_file_ok = true;
    }
    
    // [1] find a suitable image
    QString artPath = config->MPD_musicpath + sub_path;
    
    if (!b_skip_load)
    {
        QDir dir(artPath);
        if (!dir.exists())
            printf ("Album art: no such directory: " + artPath.toUtf8() + "\n");
        else
        {
            b_file_ok = go_find_art(artPath, "albumart");
            if (!b_file_ok)
                b_file_ok = go_find_art(artPath, "folder");
            if (!b_file_ok)
                b_file_ok = go_find_art(artPath, "front");
            if (!b_file_ok)
                b_file_ok = go_find_art(artPath, "cover");
            if (!b_file_ok)
                b_file_ok = go_find_art(artPath, ".");  // anything else?
                
            if (!b_file_ok)
                artPath == "nopic";
                
            if (current_art_path == artPath)
                return; // nothing to do
            else
                current_art_path = artPath;
                
            // [2] load the image
            if (b_file_ok)
            {
                temp.load(artPath);
                
                if (temp.isNull())
                {
                    b_file_ok = false;
                    printf ("Album art: error loading file: " + artPath.toUtf8() + "\n");
                }
            }
        }
    }
    
    if (!b_file_ok)
    {
        if (current_art_path == "nopic")
            return; // nothing to do
        else
        {
            current_art_path = "nopic";
            ;
            temp = QImage(img_nopic);
            b_skip_load = true;
        }
    }
    // resize the image
    temp = temp.scaled( 200, 200, Qt::KeepAspectRatio , Qt::SmoothTransformation );
    aArt_H = temp.height();
    aArt_W = temp.width();
    
    if (!b_skip_load) // (if file is used)
    {
        //  [3] add alpha for xfading
        if (!temp.hasAlphaChannel())
        {
            QImage aChannel = QImage(aArt_W, aArt_H, QImage::Format_RGB32);
            aChannel.fill(0xFFFFFFFF);
            temp.setAlphaChannel(aChannel);
        }
        else
        {  //  fill transparency with background color (for fading)
            for ( int j = 0; j <aArt_H; j++ )
            {
                QRgb *px = (QRgb*)temp.scanLine(j);
                for ( int k = 0; k < aArt_W; k++ )
                {
                    int a = qAlpha(*px);
                    uchar r = uchar( ( int(qRed(*px))   * a  +  cR * (255 - a)) / 255 );
                    uchar g = uchar( ( int(qGreen(*px)) * a  +  cG * (255 - a)) / 255 );
                    uchar b = uchar( ( int(qBlue(*px))  * a  +  cB * (255 - a)) / 255 );
                    *px = qRgba( r, g, b, 255);
                    px++;
                }
            }
        }
    }
    
    // [4] add border (if a file is used)
    if (!b_skip_load)
    {
        for (int  h = 0; h < aArt_H; h++)
        {
            for (int  w = 0; w < aArt_W; w++)
            {
                if (h == 0 || h == aArt_H-1 || w == 0 || w == aArt_W-1)
                {
                    temp.setPixel(w, h, 0xFF768B96);
                }
            }
        }
    }
    
    // [5] show image
    if (config->playerWin_maxmode)
    {
        pxb_albmart  = QPixmap::fromImage(temp);
        pxb_xfading = QPixmap::fromImage(temp);
        lb_aArt->setPixmap(pxb_albmart);
    }
    else
    {
        pxb_albmart  = QPixmap::fromImage(temp);
        // set alpha to 0
        for (int  h = 0; h < aArt_H; h++)
        {
            for (int  w = 0; w < aArt_W; w++)
            {
                QRgb RGB = temp.pixel(w,h);
                temp.setPixel(w, h, qRgba( qRed(RGB), qGreen(RGB), qBlue(RGB), 0));
            }
        }
        pxb_xfading = QPixmap::fromImage(temp);
        lb_aArt->setPixmap(pxb_xfading);
    }
    
    // [6] set tooltip
    if (config->show_tooltips && b_file_ok)
        lb_aArt->setToolTip("Album art");
    if (config->show_tooltips && !b_file_ok)
        lb_aArt->setToolTip("No album art");
}


bool qm_player::go_find_art(QString &artPath, QString findthis)
{
    QDir dir(artPath);
    
    dir.setFilter(QDir::Files);
    QStringList filters;
    filters << "*.jpg" << "*.png" << "*.gif" << "*.bmp" << "*.jpeg" << "*.JPG" << "*.PNG" << "*.GIF" << "*.BMP" << "*.JPEG";
    dir.setNameFilters(filters);
    dir.setSorting(QDir::Type);
    
    findthis = "*" + findthis + "*";
    QStringList listfilter;
    listfilter << findthis;
    QStringList files = dir.entryList(listfilter);
    if (files.isEmpty())
        return false;
        
    artPath.append(files.at(0));
    return true;
}


void qm_player::on_contextmenu()
{
    // no song, stream or not connected -> no menu
    if (b_nosong || b_stream || !mpd_connected)
        return;
    ctxMenu->exec(QCursor::pos());
}

void qm_player::showAlbumArt()
{
    QProcess *proc;
    proc->startDetached(config->art_viewer, QStringList() << current_art_path );
}

void qm_player::reloadTags()
{
    if (mpd_connected)
        mpdCom->updatefile(current_mpd_dir);
}

void qm_player::editTags()
{
    QProcess *proc;
    QString full_path = config->MPD_musicpath + current_mpd_dir;
    proc->startDetached(config->tag_editor, QStringList() << full_path);
}

void qm_player::openDirectory()
{
    QProcess *proc;
    QString full_path = config->MPD_musicpath + current_mpd_dir;
    proc->startDetached(config->file_manager, QStringList() << full_path);
}

void qm_player::showEvent(QShowEvent *se)
{
    if (b_reshow_browser)
        browserWin->show();
    if (b_reshow_settings)
        settingsWin->show();
    se->accept();
}


void qm_player::hideEvent(QHideEvent *he)
{
    if (browserWin->isVisible())
    {
        b_reshow_browser = true;
        browserWin->hide();
    }
    else
        b_reshow_browser = false;
        
    if (settingsWin->isVisible())
    {
        b_reshow_settings = true;
        settingsWin->hide();
    }
    else
        b_reshow_settings = false;
        
    config->playerWin_X = x();
    config->playerWin_Y = y();
    he->accept();
}

// Called when this window is closed
void qm_player::closeEvent( QCloseEvent * ce )
{
    if (!b_plReallyClose && b_useTrayIcon)
    {
        this->hide();
    }
    else
    {
        if (this->isVisible())
        {
            config->playerWin_X = x();
            config->playerWin_Y = y();
        }
        if (browserWin->isVisible())
        {
            config->browserWin_X = browserWin->x();
            config->browserWin_Y = browserWin->y();
            config->browserWin_W = browserWin->width();
            config->browserWin_H = browserWin->height();
        }
        if (settingsWin->isVisible())
        {
            config->settingsWin_X = settingsWin->x();
            config->settingsWin_Y = settingsWin->y();
            config->settingsWin_W = settingsWin->width();
            config->settingsWin_H = settingsWin->height();
        }
        
        if (config->quitMPD_onquit)
        {
            printf ("Closing MPD, as configured\n");
            QProcess *proc;
            proc->startDetached(config->onquit_mpd_command);
        }
        
        config->save_config();
        printf ("Thank you and goodbye!\n");
        qApp->quit();
    }
    
    ce->accept();
}


void qm_player::set_ontop()
{
    Qt::WindowFlags flags = windowFlags();
    
    if ( !config->always_ontop )
    {
        if ( flags & Qt::WindowStaysOnTopHint )
        {
            flags ^= Qt::WindowStaysOnTopHint;
            setWindowFlags(flags);
        }
    }
    else
        if ( !( flags & Qt::WindowStaysOnTopHint ) )
        {
            flags |= Qt::WindowStaysOnTopHint;
            setWindowFlags(flags);
        }
    show();
}

void qm_player::on_shudown()
{
    b_plReallyClose = true;
    close();
}


void qm_player::set_local_features(bool enabled)
{
    openaart->setVisible(enabled);
    edittags->setVisible(enabled);
    opendir->setVisible(enabled);
}


void qm_player::keyPressEvent(QKeyEvent *event)
{
    switch (event->key())
    {
    case Qt::Key_Q:
        {
            if (event->modifiers() & Qt::ControlModifier)
                on_shudown();
            break;
        }
    default:
        QMainWindow::keyPressEvent(event);
    }
}

qm_player::~qm_player()
{
    delete (mpdCom);
    delete (browserWin);
    delete (settingsWin);
    delete (config);
    delete (theTrayIcon);
    delete (ds_progress);
    delete (lb_time);
	delete (vol_slider);
}
