/*
*  qm_player.h
*  QUIMUP main player window
*  (c) 2008-2010 Johan Spee
*
*  This file is part of Quimup
*
*  QUIMUP is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  QUIMUP is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program. If not, see http://www.gnu.org/licenses/.
*/

#ifndef QM_PLAYER_H
#define QM_PLAYER_H

#include <QApplication>
#include <QMainWindow>
#include <QCloseEvent>
#include <QHideEvent>
#include <QShowEvent>
#include <QFrame>
#include <QLabel>
#include <QPushButton>
#include <QSlider>
#include <QImage>
#include <QPixmap>
#include <QSettings>
#include <QTimer>
#include <QSignalMapper>
#include <QPalette>
#include <QColorGroup>
#include <QDir>
#include <QProcess>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QSpacerItem>
#include <QMenu>
#include <QWidgetAction>
#include <stdlib.h>
#include "qm_clicklabel.h"
#include "qm_clickprogressbar.h"
#include "qm_config.h"
#include "qm_mpdcom.h"
#include "qm_trayicon.h"
#include "qm_widget_ids.h"
#include "qm_scroller.h"
#include "qm_settings.h"
#include "qm_browser.h"
#include "qm_songlist.h"


class qm_player : public QMainWindow
{
    Q_OBJECT
    
public:
    qm_player();
    virtual ~qm_player();
    
protected:
    void keyPressEvent(QKeyEvent *);
    
public slots:
    void on_shudown();
    
private slots:
    void on_contextmenu();
    void on_connect_sgnl(bool);
    void set_status(int, QString s = "");
    void on_Signal(int);
    void on_newSong(qm_songInfo);
    void on_newStatus(mpd_status *);
    void on_trayClick();
    void LockProgress(QMouseEvent *e);
    void unLockProgress();
    void maxiMice();
    void miniMice();
    void showAlbumArt();
    void openDirectory();
    void editTags();
    void reloadTags();
    void set_colors();
    void set_fonts();
    void set_ontop();
    void on_curentsong_deleted();
    void on_volumedial(int);
    
private:
    QWidget *main_widget;
    QVBoxLayout *vbox_all;
    
    QLabel *Display;
    qm_Scroller *titleScroller;
    QLabel *lb_extension;
    QLabel *lb_kbps;
    QLabel *lb_kHz;
    clickLabel *lb_time;
    QSlider *ds_progress;
    QSpacerItem *spacer;
    
    QWidget *center_widget;
    QHBoxLayout *hbox_center;
    QLabel *lb_aArt;
    
    QVBoxLayout *vbox_cright;
    QLabel *lb_album;
    QLabel *lb_comment;
    
    QHBoxLayout *hbox_bottom;
    QPushButton *bt_prev;
    QPushButton *bt_stop;
    QPushButton *bt_PlayPause;
    QPushButton *bt_next;
    QPushButton *bt_browser;
    QSlider *vol_slider;
    QPushButton *bt_sizer;
    QPushButton *bt_setts;
    
    qm_settings * settingsWin;
    qm_browser * browserWin;
    QMenu * ctxMenu;
    QSignalMapper *buttonMapper;
    qm_trayIcon *theTrayIcon;
    QTimer *maximizer;
    QTimer *minimizer;
    qm_config *config;
    qm_mpdCom *mpdCom;
    
    QString
    current_mpd_dir,
    current_art_path;
    
    int
    current_status,
    current_volume,
    windowH,
    alphacycler,
    aArt_H, aArt_W,
    songTotalTime,
	songPrevTime,
    cR, cG, cB,
    max_Win_H,
    max_Win_W;
    
    bool
    b_repeat,
    b_random,
    b_single,
    b_consume,
    b_xfade,
    b_minmax_xfade,
    b_minmax_busy,
    b_mpdSetsVolume,
    b_stream,
    mpd_connected,
    b_settingsWin_hidden,
    b_browserWin_hidden,
    b_nosong,
    b_lockProgress,
    b_plReallyClose,
    b_useTrayIcon,
    b_need_newSong,
    b_reshow_browser,
    b_reshow_settings;
    
    QPixmap
    pxb_albmart,
    pxb_xfading,
    pxb_led_noconn,
    pxb_led_paused,
    pxb_led_playing,
    pxb_led_idle,
    pxb_led_stopped;
    
    QImage
    img_nopic,
    img_onair,
    img_xfade_input;
    
    QIcon
    img_mn_pause,
    img_mn_play;
    
    QAction
    *openaart,
    *edittags,
    *loadtags,
    *opendir;
    
    void setupUI();
    void retranslateUI();
    void init_vars();
    void init_widgets();
    QString into_time(int);
    void set_albumart(QString);
    bool go_find_art(QString &, QString);
    void closeEvent( QCloseEvent *);
    void hideEvent(QHideEvent *);
    void showEvent(QShowEvent *);
    void set_local_features(bool);
};

#endif // QM_PLAYER_H
