/*
*  qm_playlistview.h
*  QUIMUP playlist listview
*  (c) 2008-2010 Johan Spee
*
*  This file is part of Quimup
*
*  QUIMUP is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  QUIMUP is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program. If not, see http://www.gnu.org/licenses/.
*/


#ifndef QM_PLAYLIST_H
#define QM_PLAYLIST_H
#include <QApplication>
#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QKeyEvent>
#include <QList>
#include <QListIterator>
#include <QMenu>
#include <QTime>
#include <QInputDialog>
#include <QMessageBox>
#include <QScrollBar>
#include "qm_songlist.h"
#include "qm_mpdcom.h"
#include "qm_config.h"
#include "qm_commandlist.h"
#include "qm_browser_ids.h"

#define set_id(id)  setData(0, Qt::UserRole + 1, id)
#define set_state(state)  setData(0, Qt::UserRole + 2, state)
#define set_time(time)   setData(0, Qt::UserRole + 4, time)
#define set_file(file)   setData(0, Qt::UserRole + 5, file)
#define set_type(type)   setData(0, Qt::UserRole + 6, type)
#define set_pos(pos)   setData(0, Qt::UserRole + 7, pos)

#define get_id   data(0, Qt::UserRole + 1).toInt()
#define get_state  data(0, Qt::UserRole + 2).toInt()
#define get_time  data(0, Qt::UserRole + 4).toInt()
#define get_file  data(0, Qt::UserRole + 5).toString()
#define get_type  data(0, Qt::UserRole + 6).toInt()
#define get_pos   data(0, Qt::UserRole + 7).toInt()

#define STATE_NEW   0
#define STATE_PLAYING  1
#define STATE_PLAYED  2

class qm_plistview : public QTreeWidget
{
	Q_OBJECT

public:
	qm_plistview(QWidget * parent = 0);
	virtual ~qm_plistview();
	
public slots:
	void set_font(QFont &);
	void set_newsong(int);
	void set_status(int);
	void set_mpdCom(qm_mpdCom*, bool);
	void set_config(qm_config*);
	void set_playlistModList(qm_commandList*);
	void clear_it();
	void select_it(int);
	void on_playlist_modification(bool);
	void dragLeaveEvent(QDragLeaveEvent *);
	void dragEnterEvent(QDragEnterEvent *);
	void dragMoveEvent(QDragMoveEvent *);
	void dropEvent(QDropEvent *);
	void showLine_at(int);
	void set_maxpanel(bool);
	
private slots:
	void on_mpd_newlist(qm_songList, int);
	void on_item_dclicked(QTreeWidgetItem *);
	void purge_it();
	void shuffle_it();
	void delete_it();
	void save_it();
	void save_selected();
	void startDrag(Qt::DropActions);
	
private:
	bool 
		b_streamplaying,
		mpd_connected,
		b_panel_max;
	
	QWidget *line;
	
	int 
	current_songPos,
	current_songID,
	current_status,
	plist_totaltime,
	dropBeforeIndex,
	plist_droppos;
	
	qm_mpdCom *mpdCom;
	QColor  col_default_fg,
	col_playing_fg,
	col_played_fg;
	
	qm_config *config;
	QMenu *ctxMenu;
	QString into_time(int);
	QAction *a_shuffle,
	*a_savelist,
	*a_savesel,
	*a_delsel;
	qm_commandList *thePlaylistModList;
	void contextMenuEvent(QContextMenuEvent *);
	void keyPressEvent(QKeyEvent *);
	void reNum_reID_list();
	void reSize_columns();
	
signals:
	void sgnl_plist_statistics(QString);
	void sgnl_plist_dropreceived(int);
	void sgnl_curentsong_deleted();
};

#endif //  QM_PLAYLIST_H
