/*
*  qm_settings.cpp
*  QUIMUP settings window
*  (c) 2008-2010 Johan Spee
*
*  This file is part of Quimup
*
*  QUIMUP is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  QUIMUP is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program. If not, see http://www.gnu.org/licenses/.
*/

#include "qm_settings.h"

qm_settings::qm_settings()
{
    setupUI();
    mpd_connected = false;
    b_profile_disabled = false;
    dev1_wasChecked = false;
    dev2_wasChecked = false;
    dev3_wasChecked = false;
    dev4_wasChecked = false;
    dev5_wasChecked = false;
    b_new_custom_colors = false;
}

void qm_settings::setupUI()
{
    if (objectName().isEmpty())
        setObjectName("settingsWin");
        
    QPixmap qpx;
    qpx = QPixmap(":/mn_icon.png");
    setWindowIcon(qpx);
    setWindowTitle(trUtf8("Quimup Settings") );
    
    main_widget = new QWidget;
    vbox_all = new QVBoxLayout(main_widget);
    setCentralWidget(main_widget); // 'attatches' it to the window
    
    tabWidget = new QTabWidget();
    tabWidget->setMinimumSize(QSize(356, 0));
    vbox_all->addWidget(tabWidget);
    hbox_buttons = new QHBoxLayout();
    hbox_buttons->setSpacing(6);
    hbox_buttons->setMargin(4);
    hbox_buttons->setAlignment(Qt::AlignRight);
    bt_ok = new QPushButton();
    bt_ok->setMinimumSize(QSize(110, 28));
    bt_ok->setAutoDefault(false);
    bt_ok->setDefault(false);
    bt_ok->setIcon(QIcon(":/tr_check.png"));
    hbox_buttons->addWidget(bt_ok);
    bt_cancel = new QPushButton();
    bt_cancel->setMinimumSize(QSize(110, 28));
    bt_cancel->setAutoDefault(false);
    bt_cancel->setDefault(true);
    bt_cancel->setIcon(QIcon(":/tr_redcross.png"));
    hbox_buttons->addWidget(bt_cancel);
    vbox_all->addLayout(hbox_buttons);
    // TAB CONNECT
    tab_con = new QWidget();
    vbox_connect = new QVBoxLayout(tab_con);
    vbox_connect->setSpacing(16);
    vbox_connect->setMargin(16);
    cb_autoConn = new QCheckBox();
    vbox_connect->addWidget(cb_autoConn);
    hbox_frofile_combo = new QHBoxLayout;
    cm_profiles = new QComboBox();
    cm_profiles->insertItems(0, QStringList()
                             << trUtf8("0")
                             << trUtf8("1")
                             << trUtf8("2")
                             << trUtf8("3")
                             << trUtf8("4") );
    hbox_frofile_combo->addWidget(cm_profiles);
    lb_profiles = new QLabel();
    hbox_frofile_combo->addWidget(lb_profiles);
    vbox_connect->addLayout(hbox_frofile_combo);
    gBox_connect = new QGroupBox(tab_con);
    grid_connect = new QGridLayout(gBox_connect);
    grid_connect->setSpacing(16);
    lb_profileName = new QLabel(gBox_connect);
    grid_connect->addWidget(lb_profileName, 0, 0);
    grid_connect->setMargin(16);
    le_profileName = new QLineEdit(gBox_connect);
    grid_connect->addWidget(le_profileName, 0, 1);
    lb_host = new QLabel(gBox_connect);
    grid_connect->addWidget(lb_host, 1, 0);
    le_hostText = new QLineEdit(gBox_connect);
    grid_connect->addWidget(le_hostText, 1, 1);
    lb_port = new QLabel(gBox_connect);
    grid_connect->addWidget(lb_port, 2, 0);
    le_portText = new QLineEdit(gBox_connect);
    grid_connect->addWidget(le_portText, 2, 1);
    lb_passwd = new QLabel(gBox_connect);
    grid_connect->addWidget(lb_passwd, 3, 0);
    le_passText = new QLineEdit(gBox_connect);
    le_passText->setEchoMode(QLineEdit::Password);
    grid_connect->addWidget(le_passText, 3, 1);
    lb_configDir = new QLabel(gBox_connect);
    grid_connect->addWidget(lb_configDir, 4, 0);
    le_configDir = new QLineEdit(gBox_connect);
    grid_connect->addWidget(le_configDir, 4, 1);
    vbox_connect->addWidget(gBox_connect);
    hbox_connect_button = new QHBoxLayout;
    hbox_connect_button->setAlignment(Qt::AlignCenter);
    bt_connect = new QPushButton();
    bt_connect->setMinimumSize(QSize(140, 32));
    bt_connect->setIcon(QIcon(":/mn_config.png"));
    hbox_connect_button->addWidget(bt_connect);
    vbox_connect->addLayout(hbox_connect_button);
    hbox_connect_status = new QHBoxLayout();
    hbox_connect_status->setAlignment(Qt::AlignCenter);
    hbox_connect_status->setSpacing(12);
    hbox_connect_status->setMargin(8);
    lb_connIcon = new QLabel();
    lb_connIcon->setPixmap(QPixmap( ":/con_dis.png" ));
    lb_connIcon->setFixedSize(QSize(32, 32));
    lb_connIcon->setAlignment(Qt::AlignLeft|Qt::AlignVCenter);
    hbox_connect_status->addWidget(lb_connIcon);
    lb_statusText = new QLabel();
    lb_statusText->setTextFormat(Qt::RichText);
    lb_statusText->setAlignment(Qt::AlignLeft|Qt::AlignVCenter);
    hbox_connect_status->addWidget(lb_statusText);
    vbox_connect->addLayout(hbox_connect_status);
    tabWidget->addTab(tab_con, "");
    // TAB SERVER
    tab_server = new QWidget();
    vbox_server = new QVBoxLayout(tab_server);
    vbox_server->setSpacing(10);
    vbox_server->setMargin(10);
    hbox_xfade = new QHBoxLayout();
    sb_xfade = new QSpinBox();
    sb_xfade->setMaximumSize(QSize(46, 26));
    sb_xfade->setMaximum(30);
    sb_xfade->setMinimum(0);
    sb_xfade->setSingleStep(1);
    hbox_xfade->addWidget(sb_xfade);
    lb_xfade = new QLabel();
    hbox_xfade->addWidget(lb_xfade);
    vbox_server->addLayout(hbox_xfade);
    replaygainBox = new QGroupBox;
    hbox_replaygain =  new QHBoxLayout(replaygainBox);
    hbox_replaygain->setSpacing(4);
    hbox_replaygain->setMargin(8);
    hbox_replaygain->setAlignment(Qt::AlignHCenter);
    rb_rpg_off = new QRadioButton("", this);
    hbox_replaygain->addWidget(rb_rpg_off);
    rb_rpg_track = new QRadioButton("", this);
    hbox_replaygain->addWidget(rb_rpg_track);
    rb_rpg_album = new QRadioButton("", this);
    hbox_replaygain->addWidget(rb_rpg_album);
    rb_rpg_auto = new QRadioButton("", this);
    hbox_replaygain->addWidget(rb_rpg_auto);
    vbox_server->addWidget(replaygainBox);
    rb_rpg_off->setEnabled(false);
    rb_rpg_track->setEnabled(false);
    rb_rpg_album->setEnabled(false);
    rb_rpg_auto->setEnabled(false);
    devicesBox = new QGroupBox();
    vbox_devices = new QVBoxLayout(devicesBox);
    vbox_devices->setSpacing(8);
    vbox_devices->setMargin(10);
    cb_dev1 = new QCheckBox();
    vbox_devices->addWidget(cb_dev1);
    cb_dev2 = new QCheckBox();
    vbox_devices->addWidget(cb_dev2);
    cb_dev3 = new QCheckBox();
    vbox_devices->addWidget(cb_dev3);
    cb_dev4 = new QCheckBox();
    vbox_devices->addWidget(cb_dev4);
    cb_dev5 = new QCheckBox();
    vbox_devices->addWidget(cb_dev5);
    vbox_server->addWidget(devicesBox);
    manageBox = new QGroupBox();
    grid_manage = new QGridLayout(manageBox);
    cb_startServer = new QCheckBox();
    grid_manage->addWidget(cb_startServer,0,0);
    grid_manage->setSpacing(16);
    vbox_startmpd = new QVBoxLayout();
    vbox_startmpd->setSpacing(2);
    vbox_startmpd->setMargin(0);
    lb_startcomm = new QLabel();
    vbox_startmpd->addWidget(lb_startcomm);
    le_startcommand = new QLineEdit();
    vbox_startmpd->addWidget(le_startcommand);
    grid_manage->addLayout(vbox_startmpd,0,1);
    cb_killServer = new QCheckBox();
    grid_manage->addWidget(cb_killServer,1,0);
    vbox_killmpd = new QVBoxLayout();
    vbox_killmpd->setSpacing(2);
    vbox_killmpd->setMargin(0);
    lb_killcomm = new QLabel();
    vbox_killmpd->addWidget(lb_killcomm);
    le_killcommand = new QLineEdit();
    vbox_killmpd->addWidget(le_killcommand);
    grid_manage->addLayout(vbox_killmpd,1,1);
    vbox_server->addWidget(manageBox);
    tabWidget->addTab(tab_server, "");
    // TAB CLIENT
    tab_client = new QWidget();
    vbox_client = new QVBoxLayout(tab_client);
    vbox_client->setSpacing(8);
    vbox_client->setMargin(10);
    vbox_checkboxes = new QVBoxLayout();
    vbox_checkboxes->setSpacing(10);
    vbox_checkboxes->setMargin(10);
    cb_useSysTray = new QCheckBox();
    vbox_checkboxes->addWidget(cb_useSysTray);
    cb_showTips = new QCheckBox();
    vbox_checkboxes->addWidget(cb_showTips);
    cb_sortwithyear = new QCheckBox();
    vbox_checkboxes->addWidget(cb_sortwithyear);
    cb_disable_aart = new QCheckBox();
    vbox_checkboxes->addWidget(cb_disable_aart);
    cb_ignore_the = new QCheckBox();
    vbox_checkboxes->addWidget(cb_ignore_the);
    cb_playerontop = new QCheckBox();
    vbox_checkboxes->addWidget(cb_playerontop);
    cb_auto_playfirst = new QCheckBox();
    vbox_checkboxes->addWidget(cb_auto_playfirst);
    vbox_client->addLayout(vbox_checkboxes);
    extprogsBox = new QGroupBox();
    grid_extprogs = new QGridLayout(extprogsBox);
    grid_extprogs->setSpacing(14);
    grid_extprogs->setMargin(10);
    lb_artviewer = new QLabel();
    grid_extprogs->addWidget(lb_artviewer, 0, 0);
    le_artviewer = new QLineEdit();
    grid_extprogs->addWidget(le_artviewer, 0, 1);
    lb_tageditor = new QLabel();
    grid_extprogs->addWidget(lb_tageditor, 1, 0);
    le_tageditor = new QLineEdit();
    grid_extprogs->addWidget(le_tageditor, 1, 1);
    lb_filebrowser = new QLabel();
    grid_extprogs->addWidget(lb_filebrowser, 2, 0);
    le_filebrowser = new QLineEdit();
    grid_extprogs->addWidget(le_filebrowser, 2, 1);
    vbox_client->addWidget(extprogsBox);
    tabWidget->addTab(tab_client, "");
    // TAB APPEARANCE
    tab_style = new QWidget();
    vbox_style = new QVBoxLayout(tab_style);
    vbox_style->setSpacing(10);
    vbox_style->setMargin(10);
    fontBox = new QGroupBox();
    grid_fonts = new QGridLayout(fontBox);
    grid_fonts->setSpacing(10);
    grid_fonts->setMargin(10);
    sb_titlefont = new QSpinBox();
    sb_titlefont->setMaximumSize(QSize(46, 26));
    sb_titlefont->setMaximum(14);
    sb_titlefont->setMinimum(6);
    sb_titlefont->setSingleStep(1);
    grid_fonts->addWidget(sb_titlefont,0,0);
    lb_titlefont = new QLabel();
    lb_titlefont->setTextFormat(Qt::RichText);
    grid_fonts->addWidget(lb_titlefont,0,1);
    sb_infofont = new QSpinBox();
    sb_infofont->setMaximumSize(QSize(46, 26));
    sb_infofont->setMaximum(14);
    sb_infofont->setMinimum(6);
    sb_infofont->setSingleStep(1);
    grid_fonts->addWidget(sb_infofont,1,0);
    lb_infofont = new QLabel();
    grid_fonts->addWidget(lb_infofont,1,1);
    sb_timefont = new QSpinBox();
    sb_timefont->setMaximumSize(QSize(46, 26));
    sb_timefont->setMaximum(14);
    sb_timefont->setMinimum(6);
    sb_timefont->setSingleStep(1);
    grid_fonts->addWidget(sb_timefont,2,0);
    lb_timefont = new QLabel();
    grid_fonts->addWidget(lb_timefont,2,1);
    sb_albumfont = new QSpinBox();
    sb_albumfont->setMaximumSize(QSize(46, 26));
    sb_albumfont->setMaximum(14);
    sb_albumfont->setMinimum(6);
    sb_albumfont->setSingleStep(1);
    grid_fonts->addWidget(sb_albumfont,3,0);
    lb_albumfont = new QLabel();
    grid_fonts->addWidget(lb_albumfont,3,1);
    sb_commentfont = new QSpinBox();
    sb_commentfont->setMaximumSize(QSize(46, 26));
    sb_commentfont->setMaximum(14);
    sb_commentfont->setMinimum(6);
    sb_commentfont->setSingleStep(1);
    grid_fonts->addWidget(sb_commentfont,4,0);
    lb_commentfont = new QLabel();
    grid_fonts->addWidget(lb_commentfont,4,1);
    sb_liblistfont = new QSpinBox();
    sb_liblistfont->setMaximumSize(QSize(46, 26));
    sb_liblistfont->setMaximum(14);
    sb_liblistfont->setMinimum(6);
    sb_liblistfont->setSingleStep(1);
    grid_fonts->addWidget(sb_liblistfont,5,0);
    lb_liblistfont = new QLabel();
    grid_fonts->addWidget(lb_liblistfont,5,1);
    bt_fonts_reset = new QPushButton();
    bt_fonts_reset->setMinimumSize(QSize(130, 24));
    grid_fonts->addWidget(bt_fonts_reset,5,2);
    vbox_style->addWidget(fontBox);
    colorBox = new QGroupBox();
    hbox_colors = new QHBoxLayout(colorBox);
    hbox_colors->setSpacing(24);
    hbox_colors->setMargin(10);
    vbox_colorchoice = new QVBoxLayout();
    vbox_colorchoice->setAlignment(Qt::AlignBottom);
    vbox_colorchoice->setSpacing(8);
    vbox_colorchoice->setMargin(0);
    rb_sysdef = new QRadioButton();
    vbox_colorchoice->addWidget(rb_sysdef);
    rb_qmcool = new QRadioButton();
    vbox_colorchoice->addWidget(rb_qmcool);
    rb_custom = new QRadioButton();
    vbox_colorchoice->addWidget(rb_custom);
    spacer_l = new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Expanding);
    vbox_colorchoice->addItem(spacer_l);
    hbox_colors->addLayout(vbox_colorchoice);
    vbox_colorcustom = new QVBoxLayout();
    vbox_colorcustom->setAlignment(Qt::AlignBottom);
    vbox_colorcustom->setSpacing(8);
    vbox_colorcustom->setMargin(0);
    cb_color_browser = new QCheckBox();
    vbox_colorcustom->addWidget(cb_color_browser);
    spacer_r = new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::Expanding);
    vbox_colorcustom->addItem(spacer_r);
    lb_cust_color = new QLabel();
    lb_cust_color->setFrameShape(QFrame::StyledPanel);
    lb_cust_color->setFrameShadow(QFrame::Sunken);
    lb_cust_color->setAlignment(Qt::AlignCenter);
    lb_cust_color->setTextFormat(Qt::RichText);
    lb_cust_color->setMinimumSize(QSize(130, 36));
    lb_cust_color->setAutoFillBackground(true);
    vbox_colorcustom->addWidget(lb_cust_color);
    bt_cust_color = new QPushButton();
    bt_cust_color->setMinimumSize(QSize(130, 24));
    vbox_colorcustom->addWidget(bt_cust_color);
    hbox_colors->addLayout(vbox_colorcustom);
    vbox_style->addWidget(colorBox);
    tabWidget->addTab(tab_style, "");
    // TAB ABOUT
    tb_about = new QWidget();
    vbox_about = new QVBoxLayout(tb_about);
    vbox_about->setSpacing(10);
    vbox_about->setMargin(10);
    hbox_about_top = new QHBoxLayout();
    hbox_about_top->setSpacing(10);
    hbox_about_top->setMargin(0);
    px_about = new QLabel();
    px_about->setFixedSize(32, 32);
    px_about->setPixmap(QPixmap( ":/mn_icon.png" ));
    hbox_about_top->addWidget(px_about);
    lb_version = new QLabel();
    lb_version->setTextFormat(Qt::RichText);
    lb_version->setText("<b>Quimup 1.2.0</b>");
    hbox_about_top->addWidget(lb_version);
    vbox_about->addLayout(hbox_about_top);
    txt_about = new QTextEdit();
    txt_about->setReadOnly(true);
    vbox_about->addWidget(txt_about);
    tabWidget->addTab(tb_about, "");
    
    colorpicker = new qm_colorpicker();
    QObject::connect(bt_cancel, SIGNAL(clicked()), this, SLOT(cancelChanges()));
    QObject::connect(bt_ok, SIGNAL(clicked()), this, SLOT(apply_settings()));
    QObject::connect(cm_profiles, SIGNAL(currentIndexChanged(int)), this, SLOT(set_profile(int)));
    QObject::connect(bt_connect, SIGNAL(clicked()), this, SLOT(serverConnect()));
    QObject::connect(bt_fonts_reset, SIGNAL(clicked()), this, SLOT(reset_fonsizes()));
    QObject::connect(bt_cust_color, SIGNAL(clicked()), this, SLOT(show_colorpicker()));
    QObject::connect(colorpicker, SIGNAL(sgnl_newcolors(QColor, QColor)), this, SLOT(on_new_customcolors(QColor, QColor)));
} // setupUi


void qm_settings::set_profile(int profile)
{
    if ( profile != 0 && b_profile_disabled )
    {
        lb_profileName->setEnabled(true);
        le_profileName->setEnabled(true);
        lb_passwd->setEnabled(true);
        le_passText->setEnabled(true);
        lb_port->setEnabled(true);
        le_portText->setEnabled(true);
        lb_host->setEnabled(true);
        le_hostText->setEnabled(true);
        lb_configDir->setEnabled(true);
        le_configDir->setEnabled(true);
    }
    
    switch (profile)
    {
    case 0:
        {
            le_profileName->setText("Autodetect");
            le_configDir->setText(config->MPD_cofigpath_0);
            le_portText->setText(QString::number(config->MPD_port_0));
            le_hostText->setText(config->MPD_host_0);
            le_passText->setText(config->MPD_password_0);
            
            lb_profileName->setEnabled(false);
            le_profileName->setEnabled(false);
            lb_passwd->setEnabled(false);
            le_passText->setEnabled(false);
            lb_port->setEnabled(false);
            le_portText->setEnabled(false);
            lb_host->setEnabled(false);
            le_hostText->setEnabled(false);
            lb_configDir->setEnabled(false);
            le_configDir->setEnabled(false);
            b_profile_disabled = true;
            break;
        }
    case 1:
        {
            le_profileName->setText(config->Profilename_1);
            le_configDir->setText(config->MPD_cofigpath_1);
            le_portText->setText(QString::number(config->MPD_port_1));
            le_hostText->setText(config->MPD_host_1);
            le_passText->setText(config->MPD_password_1);
            break;
        }
    case 2:
        {
            le_profileName->setText(config->Profilename_2);
            le_configDir->setText(config->MPD_cofigpath_2);
            le_portText->setText(QString::number(config->MPD_port_2));
            le_hostText->setText(config->MPD_host_2);
            le_passText->setText(config->MPD_password_2);
            break;
        }
    case 3:
        {
            le_profileName->setText(config->Profilename_3);
            le_configDir->setText(config->MPD_cofigpath_3);
            le_portText->setText(QString::number(config->MPD_port_3));
            le_hostText->setText(config->MPD_host_3);
            le_passText->setText(config->MPD_password_3);
            break;
        }
    case 4:
        {
            le_profileName->setText(config->Profilename_4);
            le_configDir->setText(config->MPD_cofigpath_4);
            le_portText->setText(QString::number(config->MPD_port_4));
            le_hostText->setText(config->MPD_host_4);
            le_passText->setText(config->MPD_password_4);
            break;
        }
    default:
        break;
    }
}


void qm_settings::retranslateUi()
{
    if (config->show_tooltips)
    {
        cb_autoConn->setToolTip(trUtf8(" Automatically connect to mpd on start up ") );
        le_portText->setToolTip(trUtf8(" Higher than 1024, usually 6600 ") );
        le_hostText->setToolTip(trUtf8(" \"localhost\" or a remote address ")) ;
        le_configDir->setToolTip(trUtf8(" Full path to MPD's configuration file. \n usually \"/home/user/.mpdconf\" or \"/etc/mpd.conf\" ")) ;
        le_killcommand->setToolTip(trUtf8("<nobr>&nbsp;\"mpd --kill\"&nbsp;</nobr>"));
        cm_profiles->setToolTip(trUtf8("Preconfigured login profiles") );
        le_startcommand->setToolTip(trUtf8("<nobr>&nbsp;Normally \"mpd\" or \"/usr/bin/mpd\"&nbsp;</nobr>"));
        bt_cust_color->setToolTip(trUtf8(" Select custom colors "));
        le_artviewer->setToolTip(trUtf8("<nobr>&nbsp;For example: \"gwenview\" or \"eog\"&nbsp;</nobr>"));
        le_tageditor->setToolTip(trUtf8("<nobr>&nbsp;For example: \"kid3\" or \"easytag\"&nbsp;</nobr>"));
        le_filebrowser->setToolTip(trUtf8("<nobr>&nbsp;For example: \"dolphin\" or \"nautilus\"&nbsp;</nobr>"));
        lb_xfade->setToolTip(trUtf8("<nobr>&nbsp;Turn crossfade on and off in the playlist window (playback options).&nbsp;</nobr>"));
        
    }
    bt_cancel->setText(trUtf8("Close"));
    bt_ok->setText(trUtf8("Apply"));
    bt_connect->setText(trUtf8("Connect"));
    lb_profiles->setText(trUtf8("Login profile"));
    lb_profileName->setText(trUtf8("Profile name:"));
    lb_passwd->setText(trUtf8("Password:"));
    lb_port->setText(trUtf8("Port:"));
    lb_host->setText(trUtf8("Host:"));
    cb_autoConn->setText(trUtf8("Autoconnect"));
    replaygainBox->setTitle(trUtf8("Replaygain mode"));
    rb_rpg_off->setText(trUtf8("Off"));
    rb_rpg_track->setText(trUtf8("Track"));
    rb_rpg_album->setText(trUtf8("Album"));
    rb_rpg_auto->setText(trUtf8("Auto"));
    devicesBox->setTitle(trUtf8("Ouput devices"));
    manageBox->setTitle(trUtf8("Manage MPD"));
    lb_killcomm->setText(trUtf8("command:"));
    le_killcommand->setText(trUtf8("mpd --kill"));
    cb_startServer->setText(trUtf8("Start\nwith Quimup "));
    lb_startcomm->setText(trUtf8("command:"));
    le_startcommand->setText(trUtf8("mpd"));
    cb_killServer->setText(trUtf8("Close\nwith Quimup "));
    lb_configDir->setText(trUtf8("MPD's config:"));
    fontBox->setTitle(trUtf8("Font sizes"));
    lb_xfade->setText(trUtf8("Cross-fade time (s). <i>Turn on-off in playlist.</i>"));
    lb_titlefont->setText(trUtf8("&#187; Artist : Title &#171;"));
    lb_infofont->setText(trUtf8("Codec properties"));
    lb_timefont->setText(trUtf8("Time [total time]"));
    lb_albumfont->setText(trUtf8("Album and year"));
    lb_commentfont->setText(trUtf8("Comment"));
    lb_liblistfont->setText(trUtf8("Library & playlist"));
    bt_fonts_reset->setText(trUtf8("Reset"));
    colorBox->setTitle(trUtf8("Colors"));
    rb_sysdef->setText(trUtf8("System default"));
    rb_qmcool->setText(trUtf8("Quimup default"));
    rb_custom->setText(trUtf8("Custom colors"));
    bt_cust_color->setText(trUtf8("Set colors"));
    lb_cust_color->setText(trUtf8("<b>Custom Colors</b>"));
    cb_useSysTray->setText(trUtf8("Close to system tray. (restart to apply)"));
    cb_showTips->setText(trUtf8("Show tooltips. (restart to apply)"));
    cb_sortwithyear->setText(trUtf8("In artist mode: Sort albums by year"));
    cb_disable_aart->setText(trUtf8("Disable album art"));
    cb_ignore_the->setText(trUtf8("Ignore leading 'The' when sorting artists"));
    cb_playerontop->setText(trUtf8("Keep player on top of other windows"));
    cb_auto_playfirst->setText(trUtf8("Start first song in a new playlist"));
    cb_color_browser->setText(trUtf8("Include Browser titles"));
    
    extprogsBox->setTitle(trUtf8("External programs"));
    lb_artviewer->setText(trUtf8("Album art viewer"));
    lb_tageditor->setText(trUtf8("Audio tag editor"));
    lb_filebrowser->setText(trUtf8("File manager"));
    tabWidget->setTabText(tabWidget->indexOf(tab_con), trUtf8("Connect"));
    tabWidget->setTabText(tabWidget->indexOf(tab_server), trUtf8("Server"));
    tabWidget->setTabText(tabWidget->indexOf(tab_client), trUtf8("Client"));
    tabWidget->setTabText(tabWidget->indexOf(tab_style), trUtf8("Style"));
    tabWidget->setTabText(tabWidget->indexOf(tb_about), trUtf8("About"));
    txt_about->setText(trUtf8("<font face=\"Sans Serif\"><p align=\"center\"><br>"
                              "Qt User Interface for the MUsic Player daemon<br><br>"
                              "\302\251 2008 - 2011 Johan Spee<br>"
                              "quimup@coonsden.com<br><br>"
                              "License"
                              "<div align=\"justify\">"
                              "<font size=\"-1\">This program is free software. You can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. This program is distributed in the hope that it will be useful, but without any warrenty; without even the implied warranty of merchantability or fitness for a particular purpose. See the GNU General Public License for details.</font><br>"
                              "</div>"
                              "<font color=\"mediumvioletred\"><b>Tips</b></font><br>"
                              "<div align=\"justify\">"
                              "Right-clicking the player brings up a menu where you can start <b>external programs</b> to view albumart, edit audio tags etc. Preferred programs can be set here, in the Client tab. Note that Quimup must be able to access MPD's music directory (and the files in it).<br><br>"
                              "Clicking the time counter in the main display toggles the <b>time mode</b>, i.e. <i>elapsed</i> and <i>remaining</i><br><br>"
                              "Clicking the middle mouse-button on the system-tray icon toggles <b>play / pause.</b><br><br>"
                              "The <b>volume</b> can be controlled with the mouse wheel when the cursor is on the system-tray icon or on the 'status led' in the main window - and by using the slider that pops up when the 'status led' is clicked.<br><br>"
                              "Pressing the spacebar in the playlist scrolls the <b>current song</b> into view.<br><br>"
                              "A <b>scrolling title</b> speeds up (quite a bit) when the mouse cursor is moved over it.<br><br>"
                              "<b>Album-art</b> must be located in the same directory with the music file. The program looks for a filename that matches <i>*albumart*</i>  <i>*folder*</i>  <i>*front*</i>  <i>*cover*</i> or <i>*</i> (in that order), and an extension that matches <i>.jpg</i>  <i>.png</i>  <i>.gif</i> or <i>.bmp</i> (in that order). Note that Quimup must be able to access MPD's music directory (and the files in it).<br><br>"
                              "<b>Selecting</b> an item in the library tree will disable the selection of subitems. A selected artist's albums, for instance, can not be selected because the artist already includes those albums.<br><br>"
                              "<b>Gnome</b> users are recommended to install a KDE-independent Qt theme (e.g. <i>qtcurve</i>). The gnome utility <i>qtconfig</i> can be used to configure Qt. <i>Quimup requires the Qt libraries only and does not depend on KDE in any way</i>.<br><br>"
                              "</div>"
                              "<b>Enjoy!</b><br><br>"
                              "</p></font>"));
} // retranslateUi


void qm_settings::reset_fonsizes()
{
    sb_titlefont->setValue(11);
    sb_infofont->setValue(9);
    sb_timefont->setValue(11);
    sb_albumfont->setValue(9);
    sb_commentfont->setValue(8);
    sb_liblistfont->setValue(10);
}


void qm_settings::cancelChanges()
{
    this->close();
}

void qm_settings::apply_settings()
{
    set_devices();
    apply_fontsizes();
    apply_colors();
    
    config->xfade_time = sb_xfade->value();
    
    config->auto_connect = cb_autoConn->isChecked();
    config->profile = cm_profiles->currentIndex();
    
    switch (config->profile)
    {
    case 1:
        {
            config->Profilename_1 = (le_profileName->text()).simplified();
            config->MPD_port_1 = (le_portText->text()).toInt();
            config->MPD_host_1 = (le_hostText->text()).simplified();
            config->MPD_password_1 = (le_passText->text()).simplified();
            config->MPD_cofigpath_1 = (le_configDir->text()).simplified();
            break;
        }
    case 2:
        {
            config->Profilename_2 = (le_profileName->text()).simplified();
            config->MPD_port_2 = (le_portText->text()).toInt();
            config->MPD_host_2 = (le_hostText->text()).simplified();
            config->MPD_password_2 = (le_passText->text()).simplified();
            config->MPD_cofigpath_2 = (le_configDir->text()).simplified();
            break;
        }
    case 3:
        {
            config->Profilename_3 = (le_profileName->text()).simplified();
            config->MPD_port_3 = (le_portText->text()).toInt();
            config->MPD_host_3 = (le_hostText->text()).simplified();
            config->MPD_password_3 = (le_passText->text()).simplified();
            config->MPD_cofigpath_3 = (le_configDir->text()).simplified();
            break;
        }
    case 4:
        {
            config->Profilename_4 = (le_profileName->text()).simplified();
            config->MPD_port_4 = (le_portText->text()).toInt();
            config->MPD_host_4 = (le_hostText->text()).simplified();
            config->MPD_password_4 = (le_passText->text()).simplified();
            config->MPD_cofigpath_4 = (le_configDir->text()).simplified();
            break;
        }
    default:
        break;
    }
    
    config->use_trayicon = cb_useSysTray->isChecked();
    config->show_tooltips = cb_showTips->isChecked();
    config->sort_albums_year = cb_sortwithyear->isChecked();
    config->disable_albumart = cb_disable_aart->isChecked();
    config->ignore_leading_the = cb_ignore_the->isChecked();
    
    if (config->always_ontop != cb_playerontop->isChecked())
    {
        config->always_ontop = cb_playerontop->isChecked();
        emit sgnl_ontopmode();
    }
    
    if (rb_rpg_off->isChecked())
        config->rpgain_mode = "off";
    else
        if (rb_rpg_track->isChecked())
            config->rpgain_mode = "track";
        else
            if (rb_rpg_album->isChecked())
                config->rpgain_mode = "album";
            else
                config->rpgain_mode = "auto";
                
    set_replaygain_mode();
    
    config->auto_playfirst = cb_auto_playfirst->isChecked();
    config->art_viewer = (le_artviewer->text()).simplified();
    config->tag_editor = (le_tageditor->text()).simplified();
    config->file_manager = (le_filebrowser->text()).simplified();
    
    config->quitMPD_onquit = cb_killServer->isChecked();
    config->onquit_mpd_command = (le_killcommand->text()).simplified();
    
    config->startMPD_onstart = cb_startServer->isChecked();
    config->onstart_mpd_command = (le_startcommand->text()).simplified();
    
	config->save_config();
}


void qm_settings::serverConnect()
{
    if (mpdCom == NULL) // !mpd_connected is no problem here
        return;
        
    config->auto_connect = cb_autoConn->isChecked();
    config->profile = cm_profiles->currentIndex();
    switch (config->profile)
    {
    case 1:
        {
            config->Profilename_1 = (le_profileName->text()).simplified();
            config->MPD_port_1 = (le_portText->text()).toInt();
            config->MPD_host_1 = (le_hostText->text()).simplified();
            config->MPD_password_1 = (le_passText->text()).simplified();
            config->MPD_cofigpath_1 = (le_configDir->text()).simplified();
            break;
        }
    case 2:
        {
            config->Profilename_2 = (le_profileName->text()).simplified();
            config->MPD_port_2 = (le_portText->text()).toInt();
            config->MPD_host_2 = (le_hostText->text()).simplified();
            config->MPD_password_2 = (le_passText->text()).simplified();
            config->MPD_cofigpath_2 = (le_configDir->text()).simplified();
            break;
        }
    case 3:
        {
            config->Profilename_3 = (le_profileName->text()).simplified();
            config->MPD_port_3 = (le_portText->text()).toInt();
            config->MPD_host_3 = (le_hostText->text()).simplified();
            config->MPD_password_3 = (le_passText->text()).simplified();
            config->MPD_cofigpath_3 = (le_configDir->text()).simplified();
            break;
        }
    case 4:
        {
            config->Profilename_4 = (le_profileName->text()).simplified();
            config->MPD_port_4 = (le_portText->text()).toInt();
            config->MPD_host_4 = (le_hostText->text()).simplified();
            config->MPD_password_4 = (le_passText->text()).simplified();
            config->MPD_cofigpath_4 = (le_configDir->text()).simplified();
            break;
        }
    default:
        break;
    }
    
    mpdCom->configure();
    mpdCom->mpd_connect();
}

void qm_settings::apply_fontsizes()
{
    bool fonts_changed = false;
    
    if (config->maintitle_fontsize != sb_titlefont->value())
    {
        config->maintitle_fontsize = sb_titlefont->value();
        fonts_changed = true;
    }
    if (config->codecinfo_fontsize != sb_infofont->value())
    {
        config->codecinfo_fontsize = sb_infofont->value();
        fonts_changed = true;
    }
    if (config->time_fontsize != sb_timefont->value())
    {
        config->time_fontsize = sb_timefont->value();
        fonts_changed = true;
    }
    if (config->album_fontsize != sb_albumfont->value())
    {
        config->album_fontsize = sb_albumfont->value();
        fonts_changed = true;
    }
    if (config->comment_fontsize != sb_commentfont->value())
    {
        config->comment_fontsize = sb_commentfont->value();
        fonts_changed = true;
    }
    if (config->liblist_fontsize != sb_liblistfont->value())
    {
        config->liblist_fontsize = sb_liblistfont->value();
        fonts_changed = true;
    }
    
    if (fonts_changed)
        emit sgnl_fonts(); // to player
}


void qm_settings::apply_colors()
{
    bool colors_changed = false;
    
    if (config->rb_sysdef_checked != rb_sysdef->isChecked())
    {
        config->rb_sysdef_checked = rb_sysdef->isChecked();
        colors_changed = true;
    }
    
    if (config->rb_qmcool_checked != rb_qmcool->isChecked())
    {
        config->rb_qmcool_checked = rb_qmcool->isChecked();
        colors_changed = true;
    }
    
    if (config->rb_custom_checked != rb_custom->isChecked())
    {
        config->rb_custom_checked = rb_custom->isChecked();
        colors_changed = true;
    }
    if (config->rb_custom_checked && b_new_custom_colors)
        colors_changed = true;
        
    if (config->color_browser != cb_color_browser->isChecked())
    {
        config->color_browser = cb_color_browser->isChecked();
        colors_changed = true;
    }
    
    if (colors_changed)
        emit sgnl_colors(); // to player
}

// set a pointer to the parent's config
void qm_settings::set_config(qm_config *conf)
{
    if (conf == NULL)
        return;
        
    config = conf;
    
    retranslateUi();
    
    QPalette pal (this->palette());
    pal.setColor(QPalette::Window, QColor(config->color_bg)); // background
    pal.setColor(QPalette::Foreground, QColor(config->color_fg)); // text
    lb_cust_color->setPalette(pal);
    
    sb_xfade->setValue(config->xfade_time);
    cb_autoConn->setChecked(config->auto_connect);
    cm_profiles->setItemText( 0, " Autodetect" );
    cm_profiles->setItemText( 1, " " + config->Profilename_1 );
    cm_profiles->setItemText( 2, " " + config->Profilename_2 );
    cm_profiles->setItemText( 3, " " + config->Profilename_3 );
    cm_profiles->setItemText( 4, " " + config->Profilename_4 );
    
    cm_profiles->setCurrentIndex(config->profile);
    set_profile(config->profile);
    
    cb_useSysTray->setChecked(config->use_trayicon);
    cb_showTips->setChecked(config->show_tooltips);
    cb_sortwithyear->setChecked(config->sort_albums_year);
    cb_disable_aart->setChecked(config->disable_albumart);
    cb_ignore_the->setChecked(config->ignore_leading_the);
    cb_playerontop->setChecked(config->always_ontop);
    cb_auto_playfirst->setChecked(config->auto_playfirst);
    
    le_artviewer->setText(config->art_viewer);
    le_tageditor->setText(config->tag_editor);
    le_filebrowser->setText(config->file_manager);
    
    cb_killServer->setChecked(config->quitMPD_onquit);
    le_killcommand->setText(config->onquit_mpd_command);
    
    cb_startServer->setChecked(config->startMPD_onstart);
    le_startcommand->setText(config->onstart_mpd_command);
    
    sb_titlefont->setValue(config->maintitle_fontsize);
    sb_infofont->setValue(config->codecinfo_fontsize);
    sb_timefont->setValue(config->time_fontsize);
    sb_albumfont->setValue(config->album_fontsize);
    sb_commentfont->setValue(config->comment_fontsize);
    sb_liblistfont->setValue(config->liblist_fontsize);
    
    cb_color_browser->setChecked(config->color_browser);
    
    if (config->rb_sysdef_checked)
        rb_sysdef->setChecked(true);
    else
        if (config->rb_qmcool_checked)
            rb_qmcool->setChecked(true);
        else
            if (config->rb_custom_checked)
                rb_custom->setChecked(true);
                
    if (config->rpgain_mode == "track")
        rb_rpg_track->setChecked(true);
    else
        if (config->rpgain_mode == "album")
            rb_rpg_album->setChecked(true);
        else
            if (config->rpgain_mode == "auto")
                rb_rpg_auto->setChecked(true);
            else
                rb_rpg_off->setChecked(true); // default
}

// set a pointer to the parent's mpdCom
void qm_settings::set_mpdCom(qm_mpdCom *con, bool state)
{
    mpd_connected = state;
    
    mpdCom = con;
    if (mpdCom == NULL || !mpd_connected)
    {
        lb_statusText->setText(trUtf8("<b>Not Connected<b>"));
        lb_connIcon->setPixmap(QPixmap( ":/con_dis.png" ));
        devicesBox->setEnabled(false);
        rb_rpg_off->setEnabled(false);
        rb_rpg_track->setEnabled(false);
        rb_rpg_album->setEnabled(false);
        rb_rpg_auto->setEnabled(false);
    }
    else
    {
        lb_statusText->setText( trUtf8("<b>Connected</b> to MPD ") + mpdCom->get_version());
        lb_connIcon->setPixmap(QPixmap( ":/con_ok.png" ));
        
        rb_rpg_off->setEnabled(true);
        rb_rpg_track->setEnabled(true);
        rb_rpg_album->setEnabled(true);
        rb_rpg_auto->setEnabled(true);
        
        set_replaygain_mode();
        
        devicesBox->setEnabled(true);
        get_devices();
    }
    
    if (!mpd_connected || !config->mpd_outputs_allowed)
    {
        cb_dev1->setEnabled(false);
        cb_dev2->setEnabled(false);
        cb_dev3->setEnabled(false);
        cb_dev4->setEnabled(false);
        cb_dev5->setEnabled(false);
    }
    else
    {
        cb_dev1->setEnabled(true);
        cb_dev2->setEnabled(true);
        cb_dev3->setEnabled(true);
        cb_dev4->setEnabled(true);
        cb_dev5->setEnabled(true);
    }
}


void qm_settings::set_replaygain_mode()
{
    if (!mpd_connected)
        return;
        
    if (config->rpgain_mode == "off" )
    {
        mpdCom->setReplayGainMode(0);
        if (!mpdCom->getReplayGainMode("off"))
            printf ("Replaygain mode not set correctly!\n");
        else
            printf ("Replaygain mode set to \"off\".\n");
    }
    else
        if (config->rpgain_mode == "track" )
        {
            mpdCom->setReplayGainMode(1);
            if (!mpdCom->getReplayGainMode("track"))
                printf ("Replaygain mode not set correctly!\n");
            else
                printf ("Replaygain mode set to \"track\".\n");
        }
        else
            if (config->rpgain_mode == "album" )
            {
                mpdCom->setReplayGainMode(2);
                if (!mpdCom->getReplayGainMode("album"))
                    printf ("Replaygain mode not set correctly!\n");
                else
                    printf ("Replaygain mode set to \"album\".\n");
            }
            else
                if (config->rpgain_mode == "auto" )
                {
                    mpdCom->setReplayGainMode(3);
                    if (!mpdCom->getReplayGainMode("auto"))
                        printf ("Replaygain mode not set correctly!\n");
                    else
                        printf ("Replaygain mode set to \"auto\".\n");
                }
                else
                    printf ("Replaygain not set by Quimup. MPD rules.\n");
}

void qm_settings::set_devices()
{
    if (!mpd_connected)
        return;
        
    bool devs_changed = false;
    qm_outputList outputlist;
    
    // dev1
    if (cb_dev1->isEnabled())
    {
        qm_output output;
        output.id = 0;
        output.enabled = cb_dev1->isChecked();
        outputlist.push_back(output);
        if (cb_dev1->isChecked()!= dev1_wasChecked)
        {
            devs_changed = true;
            dev1_wasChecked = cb_dev1->isChecked();
        }
    }
    // dev2
    if (cb_dev2->isEnabled() )
    {
        qm_output output;
        output.id = 1;
        output.enabled = cb_dev2->isChecked();
        outputlist.push_back(output);
        if (cb_dev2->isChecked()!= dev2_wasChecked)
        {
            devs_changed = true;
            dev2_wasChecked = cb_dev2->isChecked();
        }
    }
    // dev3
    if (cb_dev3->isEnabled())
    {
        qm_output output;
        output.id = 2;
        output.enabled = cb_dev3->isChecked();
        outputlist.push_back(output);
        if (cb_dev3->isChecked()!= dev3_wasChecked)
        {
            devs_changed = true;
            dev3_wasChecked = cb_dev3->isChecked();
        }
    }
    // dev4
    if (cb_dev4->isEnabled())
    {
        qm_output output;
        output.id = 3;
        output.enabled = cb_dev4->isChecked();
        outputlist.push_back(output);
        if (cb_dev4->isChecked()!= dev4_wasChecked)
        {
            devs_changed = true;
            dev4_wasChecked = cb_dev4->isChecked();
        }
    }
    // dev5
    if (cb_dev5->isEnabled())
    {
        qm_output output;
        output.id = 4;
        output.enabled = cb_dev5->isChecked();
        outputlist.push_back(output);
        if (cb_dev5->isChecked()!= dev5_wasChecked)
        {
            devs_changed = true;
            dev5_wasChecked = cb_dev5->isChecked();
        }
    }
    
    if (devs_changed)
        mpdCom->setOutputs(outputlist);
}

// set the output devices
void qm_settings::get_devices()
{
    if (!mpd_connected)
        return;
        
    qm_outputList outputlist;
    qm_output output;
    dev1_wasChecked = false;
    dev2_wasChecked = false;
    dev3_wasChecked = false;
    dev4_wasChecked = false;
    dev5_wasChecked = false;
    outputlist = mpdCom->getOutputs();
    // Set up the output devices
    QListIterator<qm_output> i(outputlist);
    
    if (i.hasNext())
    {
        cb_dev1->setCheckable(true);
        output = i.next();
        cb_dev1->setText(output.name);
        cb_dev1->setChecked(output.enabled);
        dev1_wasChecked = output.enabled;
    }
    else
    {
        cb_dev1->setCheckable(false);
        cb_dev1->setChecked(false);
        cb_dev1->setText( trUtf8("Not connected!") );
        cb_dev2->setCheckable(false);
        cb_dev2->setChecked(false);
        cb_dev2->setText(trUtf8("n.a."));
        cb_dev3->setCheckable(false);
        cb_dev3->setChecked(false);
        cb_dev3->setText(trUtf8("n.a."));
        cb_dev4->setCheckable(false);
        cb_dev4->setChecked(false);
        cb_dev4->setText(trUtf8("n.a."));
        cb_dev5->setCheckable(false);
        cb_dev5->setChecked(false);
        cb_dev5->setText(trUtf8("n.a."));
        return;
    }
    
    if (i.hasNext())
    {
        cb_dev2->setCheckable(true);
        output = i.next();
        cb_dev2->setText(output.name);
        cb_dev2->setChecked(output.enabled);
        dev2_wasChecked = output.enabled;
    }
    else
    {
        cb_dev2->setCheckable(false);
        cb_dev2->setChecked(false);
        cb_dev2->setText(trUtf8("n.a."));
        cb_dev3->setCheckable(false);
        cb_dev3->setChecked(false);
        cb_dev3->setText(trUtf8("n.a."));
        cb_dev4->setCheckable(false);
        cb_dev4->setChecked(false);
        cb_dev4->setText(trUtf8("n.a."));
        cb_dev5->setCheckable(false);
        cb_dev5->setChecked(false);
        cb_dev5->setText(trUtf8("n.a."));
        return;
    }
    
    if (i.hasNext())
    {
        cb_dev3->setCheckable(true);
        output = i.next();
        cb_dev3->setText(output.name);
        cb_dev3->setChecked(output.enabled);
        dev3_wasChecked = output.enabled;
    }
    else
    {
        cb_dev3->setCheckable(false);
        cb_dev3->setChecked(false);
        cb_dev3->setText(trUtf8("n.a."));
        cb_dev4->setCheckable(false);
        cb_dev4->setChecked(false);
        cb_dev4->setText(trUtf8("n.a."));
        cb_dev5->setCheckable(false);
        cb_dev5->setChecked(false);
        cb_dev5->setText(trUtf8("n.a."));
        return;
    }
    
    if (i.hasNext())
    {
        cb_dev4->setCheckable(true);
        output = i.next();
        cb_dev4->setText(output.name);
        cb_dev4->setChecked(output.enabled);
        dev4_wasChecked = output.enabled;
    }
    else
    {
        cb_dev4->setCheckable(false);
        cb_dev4->setChecked(false);
        cb_dev4->setText(trUtf8("n.a."));
        cb_dev5->setCheckable(false);
        cb_dev5->setChecked(false);
        cb_dev5->setText(trUtf8("n.a."));
        return;
    }
    
    if (i.hasNext())
    {
        cb_dev5->setCheckable(true);
        output = i.next();
        cb_dev5->setText(output.name);
        cb_dev5->setChecked(output.enabled);
        dev5_wasChecked = output.enabled;
    }
    else
    {
        cb_dev5->setCheckable(false);
        cb_dev5->setChecked(false);
        cb_dev5->setText(trUtf8("n.a."));
        return;
    }
}


void qm_settings::closeEvent( QCloseEvent * ce )
{
    this->hide();
    ce->ignore(); // only hide, don't close
}

void qm_settings::hideEvent(QHideEvent *he)
{
    if (colorpicker->isVisible())
        colorpicker->hide();
        
    config->settingsWin_X = x();
    config->settingsWin_Y = y();
    config->settingsWin_W = width();
    config->settingsWin_H = height();
    he->accept();
}


void qm_settings::showEvent(QShowEvent *se)
{
    move(config->settingsWin_X, config->settingsWin_Y);
    se->accept();
}


void qm_settings::show_colorpicker()
{
    if (colorpicker->isVisible())
    {
        colorpicker->hide();
        return;
    }
    
    colorpicker->move(x()+160, y()+190);
    colorpicker->setColors(config->color_fg, config->color_bg);
    colorpicker->show();
}


void qm_settings::on_new_customcolors(QColor fgcol, QColor bgcol)
{
    config->color_bg = bgcol.name(); // "#RRGGBB"
    config->color_fg = fgcol.name();
    QPalette pal (this->palette());
    pal.setColor(QPalette::Window, QColor(config->color_bg)); // background
    pal.setColor(QPalette::Foreground, QColor(config->color_fg)); // text
    lb_cust_color->setPalette(pal);
    b_new_custom_colors = true;
}


qm_settings::~qm_settings()
{
    delete(colorpicker);
}
