/*
*  qm_mpdcom.h
*  QUIMUP mpd communicator class
*  (c) 2008-2010 Johan Spee
*
*  This file is part of Quimup
*
*  QUIMUP is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  QUIMUP is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program. If not, see http://www.gnu.org/licenses/.
*/

#include "qm_trayicon.h"



qm_trayIcon::qm_trayIcon(QWidget *parent) : QSystemTrayIcon(parent)
{
    if (objectName().isEmpty())
        setObjectName("theTrayIcon");
        
    pxb_st_playing = QPixmap(":/st_playing");
    pxb_st_noconn = QPixmap(":/st_noconn");
    pxb_st_paused = QPixmap(":/st_paused");
    pxb_st_stopped = QPixmap(":/st_stopped");
    pxb_st_idle = QPixmap(":/st_idle");
    pxb_prev = QPixmap(":/st_prev");
    pxb_stop = QPixmap(":/st_stop");
    pxb_play = QPixmap(":/st_play");
    pxb_pause = QPixmap(":/st_pause");
    pxb_next = QPixmap(":/st_next");
    pxb_exit = QPixmap(":/st_exit");
    
    mpdCom = NULL;
    setIcon(pxb_st_noconn);
    
    QMenu *menu = new QMenu(parent);
    
    m_prevAction = new QAction(tr("&prev"), this);
    m_prevAction->setIcon(pxb_prev);
    m_prevAction->setText(tr("Previous"));
    QObject::connect(m_prevAction, SIGNAL(triggered()), this, SLOT(on_prevAction()));
    menu->addAction(m_prevAction);
    
    m_stopAction = new QAction(tr("&stop"), this);
    m_stopAction->setIcon(pxb_stop);
    m_stopAction->setText(tr("Stop"));
    QObject::connect(m_stopAction, SIGNAL(triggered()), this, SLOT(on_stopAction()));
    menu->addAction(m_stopAction);
    
    m_playpauseAction = new QAction(tr("&play"), this);
    m_playpauseAction->setIcon(pxb_play);
    m_playpauseAction->setText(tr("Play"));
    QObject::connect(m_playpauseAction, SIGNAL(triggered()), this, SLOT(on_playAction()));
    menu->addAction(m_playpauseAction);
    
    m_nextAction = new QAction(tr("&next"), this);
    m_nextAction->setIcon(pxb_next);
    m_nextAction->setText(tr("Next"));
    QObject::connect(m_nextAction, SIGNAL(triggered()), this, SLOT(on_nextAction()));
    menu->addAction(m_nextAction);
    
    menu->addSeparator();
    
    m_quitAction = new QAction(tr("&exit"), this);
    m_quitAction->setIcon(pxb_exit);
    m_quitAction->setText(tr("Quit"));
    QObject::connect(m_quitAction, SIGNAL(triggered()), this, SLOT(on_quitAction()));
    menu->addAction(m_quitAction);
    
    setContextMenu(menu);
    
    QObject::connect(this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(action(QSystemTrayIcon::ActivationReason)));
    
    mpd_connected = false;
    setToolTip("Idle");
}


void qm_trayIcon::on_prevAction()
{
    if (mpd_connected)
        emit traySignal_menu(ID_prev);
}

void qm_trayIcon::on_stopAction()
{
    if (mpd_connected)
        emit traySignal_menu(ID_stop);
}

void qm_trayIcon::on_playAction()
{
    if (mpd_connected)
        emit traySignal_menu(ID_play);
}

void qm_trayIcon::on_nextAction()
{
    if (mpd_connected)
        emit traySignal_menu(ID_next);
}

void qm_trayIcon::on_quitAction()
{
    emit traySignal_menu(ID_exit);
}

void qm_trayIcon::action(QSystemTrayIcon::ActivationReason reason)
{
    if (reason == QSystemTrayIcon::MiddleClick)
        on_playAction();
    else if (reason != QSystemTrayIcon::Context)
        emit clicked();
}

void qm_trayIcon::setTip(QString str)
{
    if (str.isNull())
    {
        setToolTip("I am a tiltoop");
        return;
    }
    else
        setToolTip(str);
}

// set a pointer to the parent's mpdCom
void qm_trayIcon::set_mpdCom(qm_mpdCom *com, bool isconnected)
{
    mpdCom = com;
    mpd_connected = isconnected;
    if (!mpd_connected)
        set_mode(ID_nocn);
    else
        set_mode(ID_idle);
}

void qm_trayIcon::set_mode(int mode)
{
    switch (mode)
    {
    case ID_play: // playing
        {
            m_playpauseAction->setIcon(pxb_pause);
            m_playpauseAction->setText(tr("Pause"));
            setIcon(pxb_st_playing);
            break;
        }
        
    case ID_paus: // paused
        {
            m_playpauseAction->setIcon(pxb_play);
            m_playpauseAction->setText(tr("Play"));
            setIcon(pxb_st_paused);
            break;
        }
        
    case ID_stop: // stopped
        {
            m_playpauseAction->setIcon(pxb_play);
            m_playpauseAction->setText(tr("Play"));
            setIcon(pxb_st_stopped);
            break;
        }
        
    case ID_nocn: // noconnection
        {
            m_playpauseAction->setIcon(pxb_play);
            m_playpauseAction->setText(tr("Play"));
            setIcon(pxb_st_noconn);
            break;
        }
        
    default:
        {
            m_playpauseAction->setIcon(pxb_play);
            m_playpauseAction->setText(tr("Play"));
            setIcon(pxb_st_idle);
            break;
        }
    }
}

bool qm_trayIcon::event(QEvent *event)
{
    if (event->type() == QEvent::Wheel && mpd_connected)
    {
        QWheelEvent *e = static_cast<QWheelEvent*>(event);
        const int numDegrees = e->delta() / 8;
        const int numSteps = numDegrees / 15;
        if (numSteps > 0 && mpdCom != NULL)
        {
            mpdCom->volume_up(5*numSteps);
        }
        else
            if (numSteps < 0  && mpdCom != NULL)
            {
                mpdCom->volume_down(-5*numSteps);
            }
        return true;
    }
    return QSystemTrayIcon::event(event);
}

qm_trayIcon::~qm_trayIcon()
{}
